from ..models.cq import reply_msg, text_msg, to_cq_str_action
from ..types.abc import ActionArgs, BotAction
from ..types.exceptions import *
from ..types.typing import *
from ..utils.atools import get_id
from .session import SESSION_LOCAL
from .session import BotSessionManager as CtxManager

if TYPE_CHECKING:
    from ..models.event import BotEvent, ResponseEvent

__all__ = [
    "send_custom_msg",
    "send",
    "send_custom_forward",
    "send_forward",
    "msg_recall",
    "get_msg",
    "get_forward_msg",
    "get_image",
    "mark_msg_read",
    "group_kick",
    "group_ban",
    "group_anonym_ban",
    "group_whole_ban",
    "set_group_admin",
    "set_group_card",
    "set_group_name",
    "group_leave",
    "set_group_title",
    "group_sign",
    "set_friend_add",
    "set_group_add",
    "get_login_info",
    "set_login_profile",
    "get_stranger_info",
    "get_friend_list",
    "get_undirect_friend",
    "delete_friend",
    "get_group_info",
    "get_group_list",
    "get_group_member_info",
    "get_group_member_list",
    "get_group_honor",
    "check_send_image",
    "check_send_record",
    "get_cq_version",
    "set_group_portrait",
    "ocr",
    "get_group_sys_msg",
    "upload_file",
    "get_group_filesys_info",
    "get_group_root_files",
    "get_group_files_byfolder",
    "create_group_folder",
    "delete_group_folder",
    "delete_group_file",
    "get_group_file_url",
    "get_cq_status",
    "get_atall_remain",
    "quick_handle",
    "set_group_notice",
    "get_group_notice",
    "download_file",
    "get_online_clients",
    "get_group_msg_history",
    "set_group_essence",
    "get_group_essence_list",
    "get_model_show",
    "set_model_show",
    "delete_undirect_friend",
    "take_custom_action",
    "send_hup",
    "send_reply",
    "finish",
    "finish_reply",
]


class MsgActionArgs(ActionArgs):
    """
    消息 action 信息构造类
    """

    def __init__(
        self,
        msgs: List[CQMsgDict],
        isPrivate: bool,
        userId: int = None,
        groupId: int = None,
    ) -> None:
        super().__init__()
        self.type = "send_msg"
        if isPrivate:
            self.params = {
                "message_type": "private",
                "user_id": userId,
                "message": msgs,
            }
        else:
            self.params = {
                "message_type": "group",
                "user_id": userId,
                "group_id": groupId,
                "message": msgs,
            }


def _process_msg(content: str | CQMsgDict | List[CQMsgDict]) -> list[CQMsgDict]:
    """
    将多种可能的消息格式，统一转换为 cq 消息列表
    """
    if isinstance(content, str):
        msgs = text_msg(content)
        if not isinstance(msgs, list):
            msgs = [msgs]
    elif isinstance(content, dict):
        msgs = [content]
    elif isinstance(content, list):
        temp = []
        for _ in content:
            if isinstance(_, list):
                temp.extend(_)
            else:
                temp.append(_)
        msgs = temp
    else:
        raise BotActionError("content 参数类型不正确，无法封装")
    return msgs


@CtxManager._activate
async def send_custom_msg(
    content: str | CQMsgDict | List[CQMsgDict],
    isPrivate: bool,
    userId: int = None,
    groupId: int = None,
    cq_str: bool = False,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    发送消息
    """
    if isPrivate and userId is None:
        raise BotActionError("为私聊时，构建 action 必须提供 userId 参数")
    if not isPrivate and groupId is None:
        raise BotActionError("为群聊时，构建 action 必须提供 groupId 参数")
    action = BotAction(
        MsgActionArgs(_process_msg(content), isPrivate, userId, groupId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )
    if cq_str:
        action = to_cq_str_action(action)
    return action


@CtxManager._activate
async def send(
    content: str | CQMsgDict | List[CQMsgDict],
    cq_str: bool = False,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    在当前 session 上下文发送一条消息
    """
    try:
        session = SESSION_LOCAL
        if len(session.events) > 0:
            action = BotAction(
                MsgActionArgs(
                    _process_msg(content),
                    session.event.is_private(),
                    session.event.sender.id,
                    session.event.group_id,
                ),
                resp_id=get_id() if wait else None,
                ready=auto,
            )
            if cq_str:
                action = to_cq_str_action(action)
            return action
        else:
            raise BotSessionError("当前 session 上下文没有事件信息，因此无法使用本方法")
    except LookupError:
        raise BotSessionError("当前作用域内 session 上下文不存在，因此无法使用本方法")


class ForwardMsgActionArgs(ActionArgs):
    """
    转发消息 action 信息构造类
    """

    def __init__(
        self,
        msgs: Dict,
        isPrivate: bool,
        userId: int = None,
        groupId: int = None,
    ) -> None:
        super().__init__()
        if isPrivate:
            self.type = "send_private_forward_msg"
            self.params = {"user_id": userId, "messages": msgs, "auto_escape": True}
        else:
            self.type = "send_group_forward_msg"
            self.params = {"group_id": groupId, "messages": msgs, "auto_escape": True}


@CtxManager._activate
async def send_custom_forward(
    msgNodes: List[MsgNodeDict],
    isPrivate: bool,
    userId: int = None,
    groupId: int = None,
    cq_str: bool = False,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    转发消息发送
    """
    action = BotAction(
        ForwardMsgActionArgs(msgNodes, isPrivate, userId, groupId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )
    if cq_str:
        action = to_cq_str_action(action)
    return action


@CtxManager._activate
async def send_forward(
    msgNodes: List[MsgNodeDict],
    cq_str: bool = False,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    在当前 session 上下文发送转发消息
    """
    try:
        session = SESSION_LOCAL
        if len(session.events) > 0:
            action = BotAction(
                ForwardMsgActionArgs(
                    msgNodes,
                    session.event.is_private(),
                    session.event.sender.id,
                    session.event.group_id,
                ),
                resp_id=get_id() if wait else None,
                ready=auto,
            )
            if cq_str:
                action = to_cq_str_action(action)
            return action
        else:
            raise BotSessionError("当前 session 上下文没有事件信息，因此无法使用本方法")
    except LookupError:
        raise BotSessionError("当前作用域内 session 上下文不存在，因此无法使用本方法")


class MsgDelActionArgs(ActionArgs):
    """
    撤回消息 action 信息构造类
    """

    def __init__(self, msgId: int) -> None:
        super().__init__()
        self.type = "delete_msg"
        self.params = {
            "message_id": msgId,
        }


@CtxManager._activate
async def msg_recall(
    msgId: int, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    撤回消息
    """
    return BotAction(
        MsgDelActionArgs(msgId), resp_id=get_id() if wait else None, ready=auto
    )


class GetMsgActionArgs(ActionArgs):
    """
    消息信息获取 action 信息构造类
    """

    def __init__(self, msgId: int) -> None:
        super().__init__()
        self.type = "get_msg"
        self.params = {"message_id": msgId}


@CtxManager._activate
async def get_msg(
    msgId: int, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取消息详细信息
    """
    return BotAction(
        GetMsgActionArgs(msgId), resp_id=get_id() if wait else None, ready=auto
    )


class getForwardActionArgs(ActionArgs):
    """
    转发消息获取 action 信息构造类
    """

    def __init__(self, forwardId: str) -> None:
        super().__init__()
        self.type = "get_forward_msg"
        self.params = {"message_id": forwardId}


@CtxManager._activate
async def get_forward_msg(
    forwardId: str, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    转发消息获取
    """
    return BotAction(
        getForwardActionArgs(forwardId), resp_id=get_id() if wait else None, ready=auto
    )


class getImageActionArgs(ActionArgs):
    """
    获取图片信息 action 信息构造类
    """

    def __init__(self, fileName: str) -> None:
        super().__init__()
        self.type = "get_image"
        self.params = {"file": fileName}


@CtxManager._activate
async def get_image(
    fileName: str, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取图片信息
    """
    return BotAction(
        getImageActionArgs(fileName), resp_id=get_id() if wait else None, ready=auto
    )


class MarkMsgReadActionArgs(ActionArgs):
    """
    标记消息已读 action 信息构造类
    """

    def __init__(self, msgId: int) -> None:
        super().__init__()
        self.type = "mark_msg_as_read"
        self.params = {"message_id": msgId}


@CtxManager._activate
async def mark_msg_read(
    msgId: int, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    标记消息已读
    """
    return BotAction(
        MarkMsgReadActionArgs(msgId), resp_id=get_id() if wait else None, ready=auto
    )


class GroupKickActionArgs(ActionArgs):
    """
    群组踢人 action 信息构造类
    """

    def __init__(self, groupId: int, userId: int, laterReject: bool = False) -> None:
        super().__init__()
        self.type = "set_group_kick"
        self.params = {
            "group_id": groupId,
            "user_id": userId,
            "reject_add_request": laterReject,
        }


@CtxManager._activate
async def group_kick(
    groupId: int,
    userId: int,
    laterReject: bool = False,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    群组踢人
    """
    return BotAction(
        GroupKickActionArgs(groupId, userId, laterReject),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GroupBanActionArgs(ActionArgs):
    """
    群组禁言 action 信息构造类
    """

    def __init__(self, groupId: int, userId: int, duration: int) -> None:
        super().__init__()
        self.type = "set_group_ban"
        self.params = {
            "group_id": groupId,
            "user_id": userId,
            "duration": duration,
        }


@CtxManager._activate
async def group_ban(
    groupId: int, userId: int, duration: int, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    群组禁言。
    duration 为 0 取消禁言
    """
    return BotAction(
        GroupBanActionArgs(groupId, userId, duration),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GroupAnonymBanActionArgs(ActionArgs):
    """
    群组匿名禁言 action 信息构造类
    """

    def __init__(self, groupId: int, anonymFlag: str, duration: int) -> None:
        super().__init__()
        self.type = "set_group_anonymous_ban"
        self.params = {
            "group_id": groupId,
            "anonymous_flag": anonymFlag,
            "duration": duration,
        }


@CtxManager._activate
async def group_anonym_ban(
    groupId: int, anonymFlag: str, duration: int, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    群组匿名禁言。
    无法取消禁言
    """
    return BotAction(
        GroupAnonymBanActionArgs(groupId, anonymFlag, duration),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GroupWholeBanActionArgs(ActionArgs):
    """
    群组全员禁言 action 信息构造类
    """

    def __init__(self, groupId: int, enable: bool) -> None:
        super().__init__()
        self.type = "set_group_whole_ban"
        self.params = {"group_id": groupId, "enable": enable}


@CtxManager._activate
async def group_whole_ban(
    groupId: int, enable: bool, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    群组全员禁言
    """
    return BotAction(
        GroupWholeBanActionArgs(groupId, enable),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class SetGroupAdminActionArgs(ActionArgs):
    """
    设置群管理员 action 信息构造类
    """

    def __init__(self, groupId: int, userId: int, enable: bool) -> None:
        super().__init__()
        self.type = "set_group_admin"
        self.params = {"group_id": groupId, "user_id": userId, "enable": enable}


@CtxManager._activate
async def set_group_admin(
    groupId: int, userId: int, enable: bool, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    设置群管理员
    """
    return BotAction(
        SetGroupAdminActionArgs(groupId, userId, enable),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class SetGroupCardActionArgs(ActionArgs):
    """
    设置群名片 action 信息构造类
    """

    def __init__(self, groupId: int, userId: int, card: str) -> None:
        super().__init__()
        self.type = "set_group_card"
        self.params = {"group_id": groupId, "user_id": userId, "card": card}


@CtxManager._activate
async def set_group_card(
    groupId: int, userId: int, card: str, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    设置群名片
    """
    return BotAction(
        SetGroupCardActionArgs(groupId, userId, card),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class SetGroupNameActionArgs(ActionArgs):
    """
    设置群名 action 信息构造类
    """

    def __init__(self, groupId: int, name: str) -> None:
        super().__init__()
        self.type = "set_group_name"
        self.params = {"group_id": groupId, "group_name": name}


@CtxManager._activate
async def set_group_name(
    groupId: int, name: str, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    设置群名 action 信息的方法
    """
    return BotAction(
        SetGroupNameActionArgs(groupId, name),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GroupLeaveActionArgs(ActionArgs):
    """
    退出群组 action 信息构造类
    """

    def __init__(self, groupId: int, isDismiss: bool) -> None:
        super().__init__()
        self.type = "set_group_leave"
        self.params = {"group_id": groupId, "is_dismiss": isDismiss}


@CtxManager._activate
async def group_leave(
    groupId: int, isDismiss: bool, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    退出群组
    """
    return BotAction(
        GroupLeaveActionArgs(groupId, isDismiss),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class SetGroupTitleActionArgs(ActionArgs):
    """
    设置群头衔 action 信息构造类
    """

    def __init__(
        self,
        groupId: int,
        userId: int,
        title: str,
        duration: int = -1,
    ) -> None:
        super().__init__()
        self.type = "set_group_special_title"
        self.params = {
            "group_id": groupId,
            "user_id": userId,
            "special_title": title,
            "duration": duration,
        }


@CtxManager._activate
async def set_group_title(
    groupId: int,
    userId: int,
    title: str,
    duration: int = -1,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    设置群头衔
    """
    return BotAction(
        SetGroupTitleActionArgs(groupId, userId, title, duration),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GroupSignActionArgs(ActionArgs):
    """
    群打卡 action 信息构造类
    """

    def __init__(self, groupId: int) -> None:
        super().__init__()
        self.type = "send_group_sign"
        self.params = {"group_id": groupId}


@CtxManager._activate
async def group_sign(
    groupId: int, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    群打卡
    """
    return BotAction(
        GroupSignActionArgs(groupId), resp_id=get_id() if wait else None, ready=auto
    )


class SetFriendAddActionArgs(ActionArgs):
    def __init__(self, addFlag: str, approve: bool, remark: str) -> None:
        """
        处理加好友请求 action 信息构造类
        """
        super().__init__()
        self.type = "set_friend_add_request"
        self.params = {"flag": addFlag, "approve": approve, "remark": remark}


@CtxManager._activate
async def set_friend_add(
    addFlag: str, approve: bool, remark: str, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    处理加好友信息。注意 remark 目前暂未实现
    """
    return BotAction(
        SetFriendAddActionArgs(addFlag, approve, remark),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class SetGroupAddActionArgs(ActionArgs):
    """
    处理加群请求 action 信息构造类
    """

    def __init__(
        self,
        addFlag: str,
        addType: Literal["add", "invite"],
        approve: bool,
        reason: str = None,
    ) -> None:
        super().__init__()
        self.type = "set_group_add_request"
        self.params = {
            "flag": addFlag,
            "sub_type": addType,
            "approve": approve,
        }
        if reason:
            self.params["reason"] = reason


@CtxManager._activate
async def set_group_add(
    addFlag: str,
    addType: Literal["add", "invite"],
    approve: bool,
    rejectReason: str = None,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    处理加群请求
    """
    return BotAction(
        SetGroupAddActionArgs(addFlag, addType, approve, rejectReason),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetLoginInfoActionArgs(ActionArgs):
    """
    获取登录号信息 action 信息构造类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type = "get_login_info"
        self.params = {}


@CtxManager._activate
async def get_login_info(
    wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获得登录号信息
    """
    return BotAction(
        GetLoginInfoActionArgs(), resp_id=get_id() if wait else None, ready=auto
    )


class SetLoginProfileActionArgs(ActionArgs):
    """
    设置登录号资料 action 信息构造类
    """

    def __init__(
        self, nickname: str, company: str, email: str, college: str, personalNote: str
    ) -> None:
        super().__init__()
        self.type = "set_qq_profile"
        self.params = {
            "nickname": nickname,
            "company": company,
            "email": email,
            "college": college,
            "personal_note": personalNote,
        }


@CtxManager._activate
async def set_login_profile(
    nickname: str,
    company: str,
    email: str,
    college: str,
    personalNote: str,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    设置登录号资料
    """
    return BotAction(
        SetLoginProfileActionArgs(nickname, company, email, college, personalNote),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetStrangerInfoActionArgs(ActionArgs):
    """
    获取陌生人信息 action 信息构造类
    """

    def __init__(self, userId: int, noCache: bool) -> None:
        super().__init__()
        self.type = "get_stranger_info"
        self.params = {"user_id": userId, "no_cache": noCache}


@CtxManager._activate
async def get_stranger_info(
    userId: int, noCache: bool, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取陌生人信息。也可以对好友使用
    """
    return BotAction(
        GetStrangerInfoActionArgs(userId, noCache),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetFriendListActionArgs(ActionArgs):
    """
    获取好友列表 action 信息构造类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type = "get_friend_list"
        self.params = {}


@CtxManager._activate
async def get_friend_list(
    wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取好友列表
    """
    return BotAction(
        GetFriendListActionArgs(), resp_id=get_id() if wait else None, ready=auto
    )


class GetUndirectFriendActionArgs(ActionArgs):
    """
    获取单向好友列表 action 信息构造类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type = "get_unidirectional_friend_list"
        self.params = {}


@CtxManager._activate
async def get_undirect_friend(
    wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取单向好友信息列表
    """
    return BotAction(
        GetUndirectFriendActionArgs(), resp_id=get_id() if wait else None, ready=auto
    )


class DeleteFriendActionArgs(ActionArgs):
    """
    删除好友 action 信息构造类
    """

    def __init__(self, userId: int) -> None:
        super().__init__()
        self.type = "delete_friend"
        self.params = {"user_id": userId}


@CtxManager._activate
async def delete_friend(
    userId: int, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    删除好友
    """
    return BotAction(
        DeleteFriendActionArgs(userId), resp_id=get_id() if wait else None, ready=auto
    )


class GetGroupInfoActionArgs(ActionArgs):
    """
    获取群信息 action 信息构造类
    """

    def __init__(self, groupId: int, noCache: bool) -> None:
        super().__init__()
        self.type = "get_group_info"
        self.params = {"group_id": groupId, "no_cache": noCache}


@CtxManager._activate
async def get_group_info(
    groupId: int, noCache: bool, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群信息。可以是未加入的群聊
    """
    return BotAction(
        GetGroupInfoActionArgs(groupId, noCache),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupListActionArgs(ActionArgs):
    """
    获取群列表 action 信息构造类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type = "get_group_list"
        self.params = {}


@CtxManager._activate
async def get_group_list(
    wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群列表。注意返回建群时间都是 0，这是不准确的。准确的建群时间可以通过 `get_group_info_action` 获得
    """
    return BotAction(
        GetGroupListActionArgs(), resp_id=get_id() if wait else None, ready=auto
    )


class GetGroupMemberInfoActionArgs(ActionArgs):
    """
    获取群成员信息 action 信息构造类
    """

    def __init__(self, groupId: int, userId: int, noCache: bool) -> None:
        super().__init__()
        self.type = "get_group_member_info"
        self.params = {"group_id": groupId, "user_id": userId, "no_cache": noCache}


@CtxManager._activate
async def get_group_member_info(
    groupId: int, userId: int, noCache: bool, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群成员信息
    """
    return BotAction(
        GetGroupMemberInfoActionArgs(groupId, userId, noCache),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupMemberListActionArgs(ActionArgs):
    """
    获取群成员列表 action 信息构造类
    """

    def __init__(self, groupId: int, noCache: bool) -> None:
        super().__init__()
        self.type = "get_group_member_list"
        self.params = {"group_id": groupId, "no_cache": noCache}


@CtxManager._activate
async def get_group_member_list(
    groupId: int, noCache: bool, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群成员列表
    """
    return BotAction(
        GetGroupMemberListActionArgs(groupId, noCache),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupHonorActionArgs(ActionArgs):
    """
    获取群荣誉信息 action 信息构造类
    """

    def __init__(
        self,
        groupId: int,
        type: Literal[
            "talkative", "performer", "legend", "strong_newbie", "emotion", "all"
        ],
    ) -> None:
        super().__init__()
        self.type = "get_group_honor_info"
        self.params = {"group_id": groupId, "type": type}


@CtxManager._activate
async def get_group_honor(
    groupId: int,
    type: Literal[
        "talkative", "performer", "legend", "strong_newbie", "emotion", "all"
    ],
    wait: bool = True,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群荣誉信息
    """
    return BotAction(
        GetGroupHonorActionArgs(groupId, type),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class CheckSendImageActionArgs(ActionArgs):
    """
    检查是否可以发送图片 action 信息构造类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type = "can_send_image"
        self.params = {}


@CtxManager._activate
async def check_send_image(
    wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    检查是否可以发送图片
    """
    return BotAction(
        CheckSendImageActionArgs(), resp_id=get_id() if wait else None, ready=auto
    )


class CheckSendRecordActionArgs(ActionArgs):
    """
    检查是否可以发送语音 action 信息构造类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type = "can_send_image"
        self.params = {}


@CtxManager._activate
async def check_send_record(
    wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    检查是否可以发送语音
    """
    return BotAction(
        CheckSendRecordActionArgs(), resp_id=get_id() if wait else None, ready=auto
    )


class GetCqVersionActionArgs(ActionArgs):
    """
    获取 go-cqhttp 版本信息 action 信息构造类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type = "get_version_info"
        self.params = {}


@CtxManager._activate
async def get_cq_version(
    wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取 go-cqhttp 版本信息
    """
    return BotAction(
        GetCqVersionActionArgs(), resp_id=get_id() if wait else None, ready=auto
    )


class SetGroupPortraitActionArgs(ActionArgs):
    """
    设置群头像 action 信息构造类
    """

    def __init__(self, groupId: int, file: str, cache: Literal[0, 1] = 0) -> None:
        super().__init__()
        self.type = "set_group_portrait"
        self.params = {"group_id": groupId, "file": file, "cache": cache}


@CtxManager._activate
async def set_group_portrait(
    groupId: int,
    file: str,
    cache: Literal[0, 1] = 0,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    设置群头像。file 参数接受本地或网络 url 和 base64 编码。
    如本地路径为：`file:///C:/Users/Richard/Pictures/1.png`。
    特别注意：目前此 API 在登录一段时间后会因 cookie 失效而失效
    """
    return BotAction(
        SetGroupPortraitActionArgs(groupId, file, cache),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class OcrActionArgs(ActionArgs):
    """
    图片 OCR action 信息构造类
    """

    def __init__(self, image: str) -> None:
        super().__init__()
        self.type = "ocr_image"
        self.params = {"image": image}


@CtxManager._activate
async def ocr(
    image: str, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    图片 OCR。image 为图片 ID
    """
    return BotAction(
        OcrActionArgs(image), resp_id=get_id() if wait else None, ready=auto
    )


class GetGroupSysMsgActionArgs(ActionArgs):
    """
    获取群系统消息 action 信息构造类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type = "get_group_system_msg"
        self.params = {}


@CtxManager._activate
async def get_group_sys_msg(
    wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群系统消息
    """
    return BotAction(
        GetGroupSysMsgActionArgs(), resp_id=get_id() if wait else None, ready=auto
    )


class UploadFileActionArgs(ActionArgs):
    """
    发送文件 action 信息构造类
    """

    def __init__(
        self,
        isPrivate: bool,
        file: str,
        name: str,
        userId: int = None,
        groupId: int = None,
        groupFolderId: str = None,
    ) -> None:
        super().__init__()
        if isPrivate:
            self.type = "upload_private_file"
            self.params = {"user_id": userId, "file": file, "name": name}
        else:
            self.type = "upload_group_file"
            self.params = {
                "group_id": groupId,
                "file": file,
                "name": name,
                "folder": groupFolderId,
            }


@CtxManager._activate
async def upload_file(
    isPrivate: bool,
    file: str,
    sendFileName: str,
    userId: int = None,
    groupId: int = None,
    groupFolderId: str = None,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    发送文件。只支持发送本地文件。
    若为群聊文件发送，不提供 folder id，则默认上传到群文件根目录。

    示例路径：`C:/users/15742/desktop/QQ图片20230108225606.jpg`。

    （若需要发送网络文件，先使用 `download_file_action()` 方法生成下载网络文件的 action，
    action 响应后文件会放于 go-cqhttp 缓存文件夹中，可直接在消息段中引用）
    """
    return BotAction(
        UploadFileActionArgs(
            isPrivate, file, sendFileName, userId, groupId, groupFolderId
        ),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupFileSysInfoActionArgs(ActionArgs):
    """
    获取群文件系统信息 action 信息构造类
    """

    def __init__(self, groupId: int) -> None:
        super().__init__()
        self.type = "get_group_file_system_info"
        self.params = {"group_id": groupId}


@CtxManager._activate
async def get_group_filesys_info(
    groupId: int, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群文件系统信息
    """
    return BotAction(
        GetGroupFileSysInfoActionArgs(groupId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupRootFilesActionArgs(ActionArgs):
    """
    获取群根目录文件列表 action 信息构造类
    """

    def __init__(self, groupId: int) -> None:
        super().__init__()
        self.type = "get_group_root_files"
        self.params = {"group_id": groupId}


@CtxManager._activate
async def get_group_root_files(
    groupId: int, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群根目录文件列表
    """
    return BotAction(
        GetGroupRootFilesActionArgs(groupId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupFilesByFolderActionArgs(ActionArgs):
    """
    获取群子目录文件列表 action 信息构造类
    """

    def __init__(self, groupId: int, folderId: str) -> None:
        super().__init__()
        self.type = "get_group_files_by_folder"
        self.params = {"group_id": groupId, "folder_id": folderId}


@CtxManager._activate
async def get_group_files_byfolder(
    groupId: int, folderId: str, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群子目录文件列表
    """
    return BotAction(
        GetGroupFilesByFolderActionArgs(groupId, folderId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class CreateGroupFolderActionArgs(ActionArgs):
    """
    创建群文件夹 action 信息构造类
    """

    def __init__(self, groupId: int, folderName: str) -> None:
        super().__init__()
        self.type = "create_group_file_folder"
        self.params = {"group_id": groupId, "name": folderName, "parent_id": "/"}


@CtxManager._activate
async def create_group_folder(
    groupId: int, folderName: str, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    创建群文件夹。注意：只能在根目录创建文件夹
    """
    return BotAction(
        CreateGroupFolderActionArgs(groupId, folderName),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class DeleteGroupFolderActionArgs(ActionArgs):
    """
    删除群文件夹 action 信息构造类
    """

    def __init__(self, groupId: int, folderId: str) -> None:
        super().__init__()
        self.type = "delete_group_folder"
        self.params = {"group_id": groupId, "folder_id": folderId}


@CtxManager._activate
async def delete_group_folder(
    groupId: int, folderId: str, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    删除群文件夹。
    """
    return BotAction(
        DeleteGroupFolderActionArgs(groupId, folderId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class DeleteGroupFileActionArgs(ActionArgs):
    """
    删除群文件 action 信息构造类
    """

    def __init__(self, groupId: int, fileId: str, fileTypeId: int) -> None:
        super().__init__()
        self.type = "delete_group_file"
        self.params = {"group_id": groupId, "file_id": fileId, "busid": fileTypeId}


@CtxManager._activate
async def delete_group_file(
    groupId: int, fileId: str, fileTypeId: int, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    删除群文件。文件相关信息通过 `get_group_root_files_action()` 或
    `get_group_files_action` 的响应获得
    """
    return BotAction(
        DeleteGroupFileActionArgs(groupId, fileId, fileTypeId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupFileUrlActionArgs(ActionArgs):
    """
    获取群文件资源链接 action 信息构造类
    """

    def __init__(self, groupId: int, fileId: str, fileTypeId: int) -> None:
        super().__init__()
        self.type = "get_group_file_url"
        self.params = {"group_id": groupId, "file_id": fileId, "busid": fileTypeId}


@CtxManager._activate
async def get_group_file_url(
    groupId: int, fileId: str, fileTypeId: int, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群文件资源链接。文件相关信息通过 `get_group_root_files_action()` 或
    `get_group_files_action` 的响应获得
    """
    return BotAction(
        GetGroupFileUrlActionArgs(groupId, fileId, fileTypeId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetCqStatusActionArgs(ActionArgs):
    """
    获取 go-cqhttp 状态 action 信息构造类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type = "get_status"
        self.params = {}


@CtxManager._activate
async def get_cq_status(
    wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取 go-cqhttp 状态
    """
    return BotAction(
        GetCqStatusActionArgs(), resp_id=get_id() if wait else None, ready=auto
    )


class GetAtAllRemainActionArgs(ActionArgs):
    """
    获取群 @全体成员 剩余次数 action 信息构造类
    """

    def __init__(self, groupId: int) -> None:
        super().__init__()
        self.type = "get_group_at_all_remain"
        self.params = {"group_id": groupId}


@CtxManager._activate
async def get_atall_remain(
    groupId: int, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群 @全体成员 剩余次数
    """
    return BotAction(
        GetAtAllRemainActionArgs(groupId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class QuickHandleActionArgs(ActionArgs):
    """
    事件快速操作 action 信息构造类
    """

    def __init__(self, contextEvent: "BotEvent", operation: dict) -> None:
        super().__init__()
        self.type = ".handle_quick_operation"
        self.params = {"context": contextEvent.raw, "operation": operation}


@CtxManager._activate
async def quick_handle(
    contextEvent: "BotEvent", operation: dict, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    事件快速操作
    """
    return BotAction(
        QuickHandleActionArgs(contextEvent, operation),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class SetGroupNoticeActionArgs(ActionArgs):
    """
    发送群公告 action 信息构造类
    """

    def __init__(self, groupId: int, content: str, imageUrl: str = None) -> None:
        super().__init__()
        self.type = "_send_group_notice"
        self.params = {
            "group_id": groupId,
            "content": content,
        }
        if imageUrl:
            self.params["image"] = imageUrl


@CtxManager._activate
async def set_group_notice(
    groupId: int,
    content: str,
    imageUrl: str = None,
    wait: bool = False,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    发送群公告。注意 `imageUrl` 只能为本地 url，示例：`file:///C:/users/15742/desktop/123.jpg`
    """
    return BotAction(
        SetGroupNoticeActionArgs(groupId, content, imageUrl),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupNoticeActionArgs(ActionArgs):
    """
    获取群公告 action 信息构造类
    """

    def __init__(
        self,
        groupId: int,
    ) -> None:
        super().__init__()
        self.type = "_get_group_notice"
        self.params = {
            "group_id": groupId,
        }


@CtxManager._activate
async def get_group_notice(
    groupId: int, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群公告。
    群公告图片有 id，但暂时没有下载的方法
    """
    return BotAction(
        GetGroupNoticeActionArgs(groupId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class DownloadFileActionArgs(ActionArgs):
    """
    下载文件到缓存目录 action 信息构造类
    """

    def __init__(self, fileUrl: str, useThreadNum: int, headers: dict) -> None:
        super().__init__()
        self.type = "download_file"
        self.params = {"url": fileUrl, "thread_count": useThreadNum, "headers": headers}


@CtxManager._activate
async def download_file(
    fileUrl: str,
    useThreadNum: int,
    headers: list | str,
    wait: bool = True,
    auto: bool = True,
) -> Optional["ResponseEvent"] | BotAction:
    """
    下载文件到缓存目录。`headers` 的两种格式：
    ```
    "User-Agent=YOUR_UA[\\r\\n]Referer=https://www.baidu.com"
    ```
    或
    ```python
    [
        "User-Agent=YOUR_UA",
        "Referer=https://www.baidu.com"
    ]
    ```
    """
    return BotAction(
        DownloadFileActionArgs(fileUrl, useThreadNum, headers),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetOnlineClientsActionArgs(ActionArgs):
    """
    获取当前账号在线客户端列表 action 信息构造类
    """

    def __init__(self, noCache: bool) -> None:
        super().__init__()
        self.type = "get_online_clients"
        self.params = {"no_cache": noCache}


@CtxManager._activate
async def get_online_clients(
    noCache: bool, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取当前账号在线客户端列表
    """
    return BotAction(
        GetOnlineClientsActionArgs(noCache),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupMsgHistoryActionArgs(ActionArgs):
    """
    获取群消息历史记录 action 信息构造类
    """

    def __init__(self, msgSeq: int, groupId: int) -> None:
        super().__init__()
        self.type = "get_group_msg_history"
        self.params = {"message_seq": msgSeq, "group_id": groupId}


@CtxManager._activate
async def get_group_msg_history(
    msgSeq: int, groupId: int, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取群消息历史记录
    """
    return BotAction(
        GetGroupMsgHistoryActionArgs(msgSeq, groupId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class SetGroupEssenceActionArgs(ActionArgs):
    """
    设置精华消息 action 信息构造类
    """

    def __init__(self, msgId: int, type: Literal["add", "del"]) -> None:
        super().__init__()
        if type == "add":
            self.type = "set_essence_msg"
        else:
            self.type = "delete_essence_msg"
        self.params = {"message_id": msgId}


@CtxManager._activate
async def set_group_essence(
    msgId: int, type: Literal["add", "del"], wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    设置精华消息
    """
    return BotAction(
        SetGroupEssenceActionArgs(msgId, type),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetGroupEssenceListActionArgs(ActionArgs):
    """
    获取精华消息列表 action 信息构造类
    """

    def __init__(self, groupId: int) -> None:
        super().__init__()
        self.type = "get_essence_msg_list"
        self.params = {"group_id": groupId}


@CtxManager._activate
async def get_group_essence_list(
    groupId: int, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取精华消息列表
    """
    return BotAction(
        GetGroupEssenceListActionArgs(groupId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class GetModelShowActionArgs(ActionArgs):
    """
    获取在线机型 action 信息构造类
    """

    def __init__(self, model: str) -> None:
        super().__init__()
        self.type = "_get_model_show"
        self.params = {"model": model}


@CtxManager._activate
async def get_model_show(
    model: str, wait: bool = True, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    获取在线机型
    """
    return BotAction(
        GetModelShowActionArgs(model), resp_id=get_id() if wait else None, ready=auto
    )


class SetModelShowActionArgs(ActionArgs):
    """
    设置在线机型 action 信息构造类
    """

    def __init__(self, model: str, modelShow: str) -> None:
        super().__init__()
        self.type = "_set_model_show"
        self.params = {"model": model, "model_show": modelShow}


@CtxManager._activate
async def set_model_show(
    model: str, modelShow: str, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    设置在线机型
    """
    return BotAction(
        SetModelShowActionArgs(model, modelShow),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


class DeleteUndirectFriendActionArgs(ActionArgs):
    """
    删除单向好友 action 信息构造类
    """

    def __init__(self, userId: int) -> None:
        super().__init__()
        self.type = "delete_unidirectional_friend"
        self.params = {"user_id": userId}


@CtxManager._activate
async def delete_undirect_friend(
    userId: int, wait: bool = False, auto: bool = True
) -> Optional["ResponseEvent"] | BotAction:
    """
    删除单向好友
    """
    return BotAction(
        DeleteUndirectFriendActionArgs(userId),
        resp_id=get_id() if wait else None,
        ready=auto,
    )


@CtxManager._activate
async def take_custom_action(action: BotAction) -> Optional["ResponseEvent"]:
    """
    直接发送提供的 action
    """
    action.ready = True
    return action


async def send_hup(
    content: str | CQMsgDict | List[CQMsgDict],
    cq_str: bool = False,
    overtime: int = None,
) -> None:
    """
    回复一条消息然后挂起
    """
    await send(content, cq_str)
    await SESSION_LOCAL.hup(overtime)


async def send_reply(
    content: str | CQMsgDict | List[CQMsgDict],
    cq_str: bool = False,
    wait: bool = False,
) -> Optional["ResponseEvent"]:
    """
    发送一条回复消息（以回复消息的形式，回复 event 所指向的那条消息）
    """
    try:
        if len(SESSION_LOCAL.events) > 0:
            content_arr = [reply_msg(SESSION_LOCAL.event.id)]
        else:
            raise BotSessionError("当前 session 上下文没有 event，因此无法使用本方法")
    except LookupError:
        raise BotSessionError("当前作用域内 session 上下文不存在，因此无法使用本方法")

    if isinstance(content, str):
        content_arr.append(text_msg(content))
    elif isinstance(content, dict):
        content_arr.append(content)
    else:
        content_arr.extend(content)
    return await send(content_arr, cq_str, wait)


async def finish(
    content: str | CQMsgDict | List[CQMsgDict], cq_str: bool = False
) -> None:
    """
    发送一条消息，然后直接结束当前事件处理方法
    """
    await send(content, cq_str)
    SESSION_LOCAL.destory()
    raise DirectRetSignal("事件处理方法被安全地递归 return，请无视这个异常")


async def finish_reply(
    content: str | CQMsgDict | List[CQMsgDict], cq_str: bool = False
) -> Optional["ResponseEvent"]:
    """
    发送一条回复消息（以回复消息的形式，回复 event 所指向的那条消息），
    然后直接结束当前事件处理方法
    """
    try:
        if len(SESSION_LOCAL.events) > 0:
            content_arr = [reply_msg(SESSION_LOCAL.event.id)]
        else:
            raise BotSessionError("当前 session 上下文没有 event，因此无法使用本方法")
    except LookupError:
        raise BotSessionError("当前作用域内 session 上下文不存在，因此无法使用本方法")

    if isinstance(content, str):
        content_arr.append(text_msg(content))
    elif isinstance(content, dict):
        content_arr.append(content)
    else:
        content_arr.extend(content)
    await send(content_arr, cq_str)
    SESSION_LOCAL.destory()
    raise DirectRetSignal("事件处理方法被安全地递归 return，请无视这个异常")



from .atools import (
    RWController,
    cooldown,
    get_id,
    get_twin_event,
    lock,
    semaphore,
    this_dir,
    to_async,
    to_coro,
)
from .checker import GroupMsgLvl, MsgCheckerGen, MsgLvlChecker, PrivateMsgLvl
from .formatter import ArgFormatter
from .matcher import (
    AlwaysMatch,
    ContainMatch,
    EndMatch,
    FullMatch,
    RegexMatch,
    StartMatch,
)
from .parser import CmdParser, CmdParserGen



import asyncio
import sys

sys.path.append(r"E:\projects\Python\git-proj\melobot")
from melobot.models.session import BotSession, SessionLocal

local = SessionLocal()
local: BotSession


async def main():
    print(local.timestamp)


asyncio.run(main())



import asyncio as aio
import traceback
from asyncio import iscoroutinefunction
from functools import partial
from types import MethodType

from ..context.session import SESSION_LOCAL, BotSessionManager
from ..types.abc import PluginSignalHandlerArgs, ShareObjCbArgs
from ..types.exceptions import *
from ..types.typing import *

if TYPE_CHECKING:
    from ..plugin.plugin import Plugin
    from ..utils.logger import Logger


class ShareObject:
    """
    共享对象
    """

    def __init__(self, namespace: str, id: str) -> None:
        self.__space = namespace
        self.__id = id
        self.__reflect: Callable[[None], object] = lambda: None
        self.__callback: Coroutine

        self.__cb_set = aio.Event()

    def _fill_ref(self, reflect_func: Callable[[None], object]) -> None:
        self.__reflect = reflect_func

    def _fill_cb(self, callback: Coroutine) -> None:
        self.__callback = callback
        self.__cb_set.set()

    @property
    def val(self) -> Any:
        """
        共享对象引用的值
        """
        return self.__reflect()

    async def affect(self, *args, **kwargs) -> Any:
        """
        触发共享对象的回调，回调未设置时会等待。
        如果本来就没有回调，则会陷入无休止等待
        """
        await self.__cb_set.wait()
        return await self.__callback(*args, **kwargs)


class PluginStore:
    """
    插件共享存储
    """

    __store__: Dict[str, Dict[str, ShareObject]] = {}

    @classmethod
    def _create_so(
        cls, property: Optional[str], namespace: str, id: str, plugin: "Plugin"
    ) -> None:
        """
        创建共享对象。property 为 None 时，共享对象会引用到一个 None
        """
        if namespace not in cls.__store__.keys():
            cls.__store__[namespace] = {}
        obj = cls.__store__[namespace].get(id)
        if obj is None:
            obj = ShareObject(namespace, id)
            cls.__store__[namespace][id] = obj
        if property is not None:
            obj._fill_ref(lambda: getattr(plugin, property))
        else:
            obj._fill_ref(lambda: None)

    @classmethod
    def _bind_cb(
        cls,
        namespace: str,
        id: str,
        cb: Callable[..., Coroutine[Any, Any, Any]],
        plugin: "Plugin",
    ) -> None:
        """
        为共享对象绑定回调
        """
        if namespace not in cls.__store__.keys():
            raise ShareObjError(f"共享对象回调指定的命名空间 {namespace} 不存在")
        if id not in cls.__store__[namespace].keys():
            raise ShareObjError(
                f"共享对象回调指定的命名空间中，不存在标记为 {id} 的共享对象"
            )
        if cls.__store__[namespace][id].__cb_set.is_set():
            raise ShareObjError(
                f"{namespace} 中标记为 {id} 的共享对象已被注册过回调，拒绝再次注册"
            )
        cls.__store__[namespace][id]._fill_cb(partial(cb, plugin))

    @classmethod
    def echo(cls, namespace: str, id: str) -> Callable:
        """
        为共享对象指定回调的装饰器，用于处理外部的 affect 请求。
        绑定为回调后，不提供异步安全担保
        """

        def bind_cb(cb: Callable[..., Coroutine[Any, Any, Any]]) -> ShareObjCbArgs:
            return ShareObjCbArgs(namespace=namespace, id=id, cb=cb)

        return bind_cb

    @classmethod
    def get(cls, namespace: str, id: str) -> ShareObject:
        """
        获取共享对象，注意：共享对象不存在或暂时未初始化时会产生 None 引用。
        区别是前者会一直保持 None 引用，而后者则会随后绑定映射。
        但注意，除上述两种情况外，共享对象定义时也可引用到 None
        """
        if namespace not in cls.__store__.keys():
            cls.__store__[namespace] = {}
        if id not in cls.__store__[namespace].keys():
            cls.__store__[namespace][id] = ShareObject(namespace, id)
        return cls.__store__[namespace][id]


class PluginSignalHandler:
    """
    插件信号处理器
    """

    def __init__(
        self,
        namespace: str,
        signal: str,
        func: Callable[..., Coroutine[Any, Any, Any]],
        plugin: Optional[object],
    ) -> None:
        self._func = func
        self._plugin = plugin
        # 对应：绑定的 func 是插件类的实例方法
        if plugin:
            self.cb = partial(self._func, plugin)
        # 对应：绑定的 func 是普通函数
        else:
            self.cb = self._func
        self.namespace = namespace
        self.signal = signal


class PluginBus:
    """
    插件信号总线
    """

    __store__: Dict[str, Dict[str, PluginSignalHandler]] = {}
    __logger: "Logger"

    @classmethod
    def _bind(cls, logger: "Logger") -> None:
        """
        初始化该类，绑定全局日志器和行为响应器
        """
        cls.__logger = logger

    @classmethod
    def _register(
        cls, namespace: str, signal: str, handler: PluginSignalHandler
    ) -> None:
        """
        注册一个插件信号处理方法。由 plugin build 过程调用
        """
        if namespace not in cls.__store__.keys():
            cls.__store__[namespace] = {}
        cls.__store__[namespace][signal] = handler

    @classmethod
    def on(
        cls,
        namespace: str,
        signal: str,
        callback: Callable[..., Coroutine[Any, Any, Any]] = None,
    ):
        """
        动态地注册插件信号处理方法。callback 可以是类实例方法，也可以不是。
        callback 如果是类实例方法，请自行包裹为一个 partial 函数。

        例如你的插件类是：`A`，而你需要传递一个类实例方法：`A.xxx`，
        那么你的 callback 参数就应该是：`functools.partial(A.xxx, self)`
        """

        def make_args(
            func: Callable[..., Coroutine[Any, Any, Any]]
        ) -> PluginSignalHandlerArgs:
            return PluginSignalHandlerArgs(
                func=func, namespace=namespace, signal=signal
            )

        if callback is None:
            return make_args
        else:
            if isinstance(callback, PluginSignalHandlerArgs):
                raise PluginSignalError("已注册的插件信号处理方法不能再注册")
            if not iscoroutinefunction(callback):
                raise PluginSignalError(
                    f"插件信号处理方法 {callback.__name__} 必须为异步函数"
                )
            if (
                isinstance(callback, partial) and isinstance(callback.func, MethodType)
            ) or isinstance(callback, MethodType):
                raise PluginSignalError(
                    "callback 应该是 function，而不是 bound method。"
                )
            handler = PluginSignalHandler(namespace, signal, callback, plugin=None)
            cls._register(namespace, signal, handler)

    @classmethod
    async def _run_on_ctx(
        cls, handler: PluginSignalHandler, *args, forward: bool = False, **kwargs
    ) -> Any:
        """
        在指定的上下文下运行插件信号处理方法
        """
        if not forward:
            session = BotSessionManager.make_empty()
            token = SESSION_LOCAL._add_ctx(session)
        try:
            ret = await handler.cb(*args, **kwargs)
            return ret
        except Exception as e:
            e_name = e.__class__.__name__
            func_name = handler._func.__qualname__
            pre_str = "插件" + handler._plugin.ID if handler._plugin else "动态注册的"
            cls.__logger.error(
                f"{pre_str} 插件信号处理方法 {func_name} 发生异常：[{e_name}] {e}"
            )
            cls.__logger.debug(f"插件信号处理方法的 args: {args} kwargs：{kwargs}")
            cls.__logger.debug("异常回溯栈：\n" + traceback.format_exc().strip("\n"))
        finally:
            if not forward:
                SESSION_LOCAL._del_ctx(token)

    @classmethod
    async def emit(
        cls,
        namespace: str,
        signal: str,
        *args,
        wait: bool = False,
        forward: bool = False,
        **kwargs,
    ) -> Any:
        """
        触发一个插件信号。如果指定 wait 为 True，则会等待所有插件信号处理方法完成。
        若启用 forward，则会将 session 从信号触发处转发到信号处理处。
        但启用 forward，必须同时启用 wait。

        注意：如果你要等待返回结果，则需要指定 wait=True，否则不会等待且始终返回 None
        """
        if forward and not wait:
            raise PluginSignalError(
                "在触发插件信号处理方法时传递原始 session，wait 参数需要为 True"
            )
        if namespace not in cls.__store__.keys():
            raise PluginSignalError(
                f"插件信号命名空间 {namespace} 不存在，无法触发信号"
            )
        if signal not in cls.__store__[namespace].keys():
            return

        handler = cls.__store__[namespace][signal]
        if not wait:
            aio.create_task(cls._run_on_ctx(handler, forward=forward, *args, **kwargs))
            return None
        else:
            return await cls._run_on_ctx(handler, forward=forward, *args, **kwargs)



import asyncio as aio
import time
import traceback
from itertools import count

import websockets
import websockets.exceptions as wse

from ..types.abc import AbstractDispatcher, AbstractSender, BotLife
from ..types.exceptions import *
from ..types.typing import *

if TYPE_CHECKING:
    from ..models.event import BotEventBuilder
    from ..plugin.bot import BotHookBus
    from ..types.abc import BotAction
    from ..utils.logger import Logger


class BotLinker(AbstractSender):
    """
    Bot 连接模块通过连接适配器的代理，完成事件接收与行为发送。
    """

    def __init__(
        self,
        connect_host: str,
        connect_port: int,
        max_retry: int,
        retry_delay: int,
        send_interval: float,
        event_builder: Type["BotEventBuilder"],
        bot_bus: Type["BotHookBus"],
        logger: "Logger",
    ) -> None:
        super().__init__()
        self.url = f"ws://{connect_host}:{connect_port}"
        self.conn = None

        self.logger = logger
        self.slack = False
        self.max_retry_num = max_retry
        self.retry_delay = retry_delay if retry_delay > 0 else 0

        self._event_builder = event_builder
        self._bot_bus = bot_bus
        self._ready_signal = aio.Event()
        self._send_lock = aio.Lock()
        self._rest_time = send_interval
        self._pre_send_time = time.time()
        self._common_dispatcher: AbstractDispatcher
        self._resp_dispatcher: AbstractDispatcher

    def bind(
        self,
        common_dispatcher: AbstractDispatcher,
        resp_dispatcher: AbstractDispatcher,
    ) -> None:
        """
        绑定其他核心组件的方法。
        """
        self._common_dispatcher = common_dispatcher
        self._resp_dispatcher = resp_dispatcher
        self._ready_signal.set()

    async def _start(self) -> None:
        """
        启动连接
        """
        iterator = count(0) if self.max_retry_num < 0 else range(self.max_retry_num + 1)
        for _ in iterator:
            try:
                self.conn = await websockets.connect(self.url)
                await self.conn.recv()
                self.logger.info("与连接适配器，建立了 ws 连接")
                await self._bot_bus.emit(BotLife.CONNECTED)
                return
            except Exception as e:
                self.logger.warning(
                    f"ws 连接建立失败，{self.retry_delay}s 后自动重试。错误：{e}"
                )
                await aio.sleep(self.retry_delay)
        raise BotConnectFailed("连接重试已达最大重试次数，已放弃建立连接")

    async def _close(self) -> None:
        """
        关闭连接
        """
        await self.conn.close()
        self.logger.info("已经关闭与连接适配器的连接")

    async def __aenter__(self) -> "BotLinker":
        await self._start()
        return self

    async def __aexit__(
        self, exc_type: Exception, exc_val: str, exc_tb: ModuleType
    ) -> None:
        if exc_type == wse.ConnectionClosedError:
            self.logger.warning("连接适配器主动关闭, bot 将自动清理资源后关闭")
        await self._close()

    async def send(self, action: "BotAction") -> None:
        """
        发送一个 action 给连接适配器
        """
        await self._ready_signal.wait()
        await self._bot_bus.emit(BotLife.ACTION_PRESEND, action, wait=True)
        if self.slack:
            return

        try:
            async with self._send_lock:
                action_str = action.flatten()
                await aio.sleep(self._rest_time - (time.time() - self._pre_send_time))
                await self.conn.send(action_str)
                self._pre_send_time = time.time()
        except aio.CancelledError:
            self.logger.warning("发生一次递归重发")
            await self.send(action)
        except wse.ConnectionClosed:
            self.logger.error("与连接适配器的通信已经断开，无法再执行操作")

    async def listen(self) -> None:
        """
        从连接适配器接收一个事件，并转化为 BotEvent 对象传递给 dispatcher 处理
        """
        await self._ready_signal.wait()

        try:
            while True:
                try:
                    raw_event = await self.conn.recv()
                    if raw_event == "":
                        continue
                    event = self._event_builder.build(raw_event)
                    if event.is_resp_event():
                        aio.create_task(self._resp_dispatcher.dispatch(event))
                    else:
                        aio.create_task(self._common_dispatcher.dispatch(event))
                except wse.ConnectionClosed:
                    raise
                except Exception as e:
                    self.logger.error(
                        f"bot life_task 抛出异常：[{e.__class__.__name__}] {e}"
                    )
                    self.logger.debug(f"异常点的事件记录为：{raw_event}")
                    self.logger.debug(
                        "异常回溯栈：\n" + traceback.format_exc().strip("\n")
                    )
        except aio.CancelledError:
            self.logger.debug("bot 运行例程已停止")



import asyncio as aio
import traceback
from asyncio import Future

from ..types.abc import AbstractDispatcher, AbstractResponder
from ..types.exceptions import *
from ..types.typing import *

if TYPE_CHECKING:
    from ..models.event import ResponseEvent
    from ..types.abc import AbstractSender, BotAction
    from ..utils.logger import Logger


class BotResponder(AbstractResponder, AbstractDispatcher):
    """
    bot 响应模块，是 action 发送方和 bot 连接模块的媒介。
    提供 action 发送、响应回送功能
    """

    def __init__(self, logger: "Logger") -> None:
        super().__init__()
        self._resp_table: Dict[str, Future["ResponseEvent"]] = {}
        self.logger = logger

        self._ready_signal = aio.Event()
        self._action_sender: "AbstractSender"

    def bind(self, action_sender: "AbstractSender") -> None:
        """
        绑定其他核心组件的方法。独立出来，方便上层先创建实例再调用
        """
        self._action_sender = action_sender
        self._ready_signal.set()

    async def dispatch(self, resp: "ResponseEvent") -> None:
        await self._ready_signal.wait()

        try:
            if resp.id is None:
                return
            else:
                resp_fut = self._resp_table.get(resp.id)
                if resp_fut:
                    resp_fut.set_result(resp)
                    self._resp_table.pop(resp.id)
                else:
                    self.logger.error(f"收到了不匹配的携带 id 的响应：{resp.raw}")
        except Exception as e:
            self.logger.error(
                f"bot responder.dispatch 抛出异常：[{e.__class__.__name__}] {e}"
            )
            self.logger.debug(f"异常点的响应记录为：{resp.raw}")
            self.logger.debug("异常回溯栈：\n" + traceback.format_exc().strip("\n"))

    async def take_action(self, action: "BotAction") -> None:
        """
        响应器发送 action, 不等待响应
        """
        await self._ready_signal.wait()
        await self._action_sender.send(action)

    async def take_action_wait(self, action: "BotAction") -> Future["ResponseEvent"]:
        """
        响应器发送 action，并返回一个 Future 用于等待响应
        """
        await self._ready_signal.wait()
        fut = Future()
        self._resp_table[action.resp_id] = fut
        await self._action_sender.send(action)
        return fut



import importlib.util
import inspect
import os
import pathlib
import sys
from asyncio import iscoroutinefunction
from pathlib import Path

from ..context.session import BotSessionManager
from ..types.abc import (
    BotHookRunnerArgs,
    LogicMode,
    PluginSignalHandlerArgs,
    ShareObjArgs,
    ShareObjCbArgs,
)
from ..types.exceptions import *
from ..types.typing import *
from ..utils.checker import (
    AtChecker,
    FriendReqChecker,
    GroupReqChecker,
    NoticeTypeChecker,
)
from ..utils.logger import PrefixLogger
from ..utils.matcher import ContainMatch, EndMatch, FullMatch, RegexMatch, StartMatch
from .bot import BotHookBus, HookRunner, PluginProxy
from .handler import (
    AllEventHandler,
    EventHandler,
    EventHandlerArgs,
    MetaEventHandler,
    MsgEventHandler,
    NoticeEventHandler,
    ReqEventHandler,
)
from .ipc import PluginBus, PluginSignalHandler, PluginStore

if TYPE_CHECKING:
    from ..types.abc import AbstractResponder, BotLife, SessionRule
    from ..utils.checker import BotChecker
    from ..utils.logger import Logger
    from ..utils.matcher import BotMatcher
    from ..utils.parser import BotParser


class PluginLoader:
    """
    插件加载器
    """

    @classmethod
    def load_from_dir(cls, plugin_path: str) -> tuple["Plugin", str]:
        """
        从指定插件目录加载插件
        """
        if not os.path.exists(os.path.join(plugin_path, "__init__.py")):
            raise PluginLoadError(
                f"{plugin_path} 缺乏入口主文件 __init__.py，插件无法加载"
            )
        plugin_name = os.path.basename(plugin_path)
        plugins_folder = str(pathlib.Path(plugin_path).parent.resolve(strict=True))
        plugins_folder_name = os.path.basename(plugins_folder)
        if plugins_folder not in sys.path:
            importlib.import_module(plugins_folder_name)
            sys.path.insert(0, plugins_folder)
        module = importlib.import_module(
            f"{plugins_folder_name}.{plugin_name}", f"{plugins_folder_name}"
        )

        plugin_class = None
        for obj in module.__dict__.values():
            if (
                isinstance(obj, type)
                and issubclass(obj, Plugin)
                and obj.__name__ != Plugin.__name__
            ):
                plugin_class = obj
                break
        if plugin_class is None:
            raise PluginLoadError(
                "指定的入口主文件中，未发现继承 Plugin 的插件类，无法加载插件"
            )
        plugin = plugin_class()
        file_path = inspect.getfile(module)
        return (plugin, file_path)

    @classmethod
    def load_from_type(cls, _class: Type["Plugin"]) -> tuple["Plugin", str]:
        """
        从插件类对象加载插件
        """
        plugin = _class()
        file_path = inspect.getfile(_class)
        return (plugin, file_path)

    @classmethod
    def load(
        cls,
        target: str | Type["Plugin"],
        logger: "Logger",
        responder: "AbstractResponder",
    ) -> "Plugin":
        """
        加载插件
        """
        if isinstance(target, str):
            plugin, file_path = cls.load_from_dir(target)
        else:
            plugin, file_path = cls.load_from_type(target)
        root_path = pathlib.Path(file_path).parent.resolve(strict=True)
        plugin._build(root_path, logger, responder)
        return plugin


class Plugin:
    """
    bot 插件基类。所有自定义插件必须继承该类实现。
    """

    def __init__(self) -> None:
        # 插件 id 和 version 标记
        self.ID: str = self.__class__.__name__
        # 插件版本标记
        self.VERSION: str = "1.0.0"
        # 插件类所在的文件路径，PosicPath 对象
        self.PATH: Path
        # 标记了该类有哪些实例属性需要被共享。元素是共享对象构造参数或字符串
        self.SHARES: List[ShareObjArgs | str] = []
        # 被二次包装的全局日志器
        self.LOGGER: PrefixLogger

        self._handlers: List[EventHandler]
        self._proxy: PluginProxy

    def _build(
        self, root_path: Path, logger: "Logger", responder: "AbstractResponder"
    ) -> None:
        """
        初始化当前插件
        """
        self.PATH = root_path
        self.LOGGER = PrefixLogger(logger, self.ID)

        attrs_map = {
            k: v for k, v in inspect.getmembers(self) if not k.startswith("__")
        }
        share_objs_map = self._init_share_objs(attrs_map, self.SHARES)

        self._handlers = []
        signal_methods_map = []
        members = inspect.getmembers(self)
        for attr_name, val in members:
            if isinstance(val, EventHandlerArgs):
                handler = self._init_event_handler(
                    responder, logger, val[0], val[1], *val[2]
                )
                self._handlers.append(handler)
            elif isinstance(val, BotHookRunnerArgs):
                self._init_hook_runner(val.func, val.type)
            elif isinstance(val, ShareObjCbArgs):
                self._init_share_obj_cb(val.namespace, val.id, val.cb)
            elif isinstance(val, PluginSignalHandlerArgs):
                self._init_plugin_signal_handler(val.namespace, val.signal, val.func)
                signal_methods_map.append((val.namespace, val.signal))

        self._proxy = PluginProxy(
            self.ID, self.VERSION, self.PATH, share_objs_map, signal_methods_map
        )

    def _init_share_objs(
        self, attrs_map: dict[str, Any], shares: List[ShareObjArgs | str]
    ) -> list[tuple[str, str]]:
        """
        初始化所有的共享对象。
        同时返回共享对象定义时，包含 (namespace, id) 元组的列表
        """
        share_objs = []
        for idx, val in enumerate(shares):
            if isinstance(val, str):
                shares[idx] = ShareObjArgs(val, self.ID, val)
        for share_obj in shares:
            property, namespace, id = (
                share_obj.property,
                share_obj.namespace,
                share_obj.id,
            )
            if property not in attrs_map.keys() and property is not None:
                raise PluginBuildError(
                    f"插件 {self.ID} 尝试共享一个不存在的属性 {property}"
                )
            PluginStore._create_so(property, namespace, id, self)
            share_objs.append((namespace, id))
        return share_objs

    def _init_event_handler(
        self,
        responder: "AbstractResponder",
        logger: "Logger",
        executor: Callable[[], Coroutine[Any, Any, None]],
        handler_class: Type[EventHandler],
        *params: Any,
    ) -> EventHandler:
        """
        初始化指定的 event handler，并返回
        """
        if not iscoroutinefunction(executor):
            raise PluginBuildError(f"事件处理器 {executor.__name__} 必须为异步方法")
        overtime_cb_maker, conflict_cb_maker = params[-1], params[-2]
        if overtime_cb_maker and not callable(overtime_cb_maker):
            raise PluginBuildError(
                f"超时回调方法 {overtime_cb_maker.__name__} 必须为可调用对象"
            )
        if conflict_cb_maker and not callable(conflict_cb_maker):
            raise PluginBuildError(
                f"冲突回调方法 {conflict_cb_maker.__name__} 必须为可调用对象"
            )
        handler = handler_class(executor, self, responder, logger, *params)
        BotSessionManager.register(handler)
        return handler

    def _init_hook_runner(
        self, hook_func: Callable[..., Coroutine[Any, Any, None]], type: "BotLife"
    ) -> None:
        """
        初始化指定的 bot 生命周期的 hook 方法
        """
        if not iscoroutinefunction(hook_func):
            raise PluginBuildError(f"hook 方法 {hook_func.__name__} 必须为异步函数")
        runner = HookRunner(type, hook_func, self)
        BotHookBus._register(type, runner)

    def _init_share_obj_cb(
        self, namespace: str, id: str, cb: Callable[..., Coroutine[Any, Any, Any]]
    ) -> None:
        """
        初始化指定的共享对象的回调
        """
        if not iscoroutinefunction(cb):
            raise PluginBuildError(
                f"{namespace} 命名空间中，id 为 {id} 的共享对象，它的回调 {cb.__name__} 必须为异步函数"
            )
        PluginStore._bind_cb(namespace, id, cb, self)

    def _init_plugin_signal_handler(
        self, namespace: str, signal: str, func: Callable[..., Coroutine[Any, Any, Any]]
    ) -> None:
        """
        初始化指定的插件信号处理方法
        """
        if not iscoroutinefunction(func):
            raise PluginBuildError(f"插件信号处理方法 {func.__name__} 必须为异步函数")
        handler = PluginSignalHandler(namespace, signal, func, self)
        PluginBus._register(namespace, signal, handler)

    @classmethod
    def on_event(
        cls,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为任意事件处理器（响应事件除外）
        """

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=AllEventHandler,
                params=[
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_message(
        cls,
        matcher: "BotMatcher" = None,
        parser: "BotParser" = None,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为消息事件处理器
        """

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=MsgEventHandler,
                params=[
                    matcher,
                    parser,
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_every_message(
        cls,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为任意消息事件处理器。
        任何消息经过校验后，不进行匹配和解析即可触发处理方法
        """

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=MsgEventHandler,
                params=[
                    None,
                    None,
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_at_qq(
        cls,
        qid: int = None,
        matcher: "BotMatcher" = None,
        parser: "BotParser" = None,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为艾特消息匹配的消息事件处理器。
        必须首先是来自指定 qid 的艾特消息，才能被进一步处理
        """
        at_checker = AtChecker(qid)
        if checker is not None:
            wrapped_checker = at_checker & checker
        else:
            wrapped_checker = at_checker

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=MsgEventHandler,
                params=[
                    matcher,
                    parser,
                    wrapped_checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_start_match(
        cls,
        target: str | list[str],
        logic_mode: LogicMode = LogicMode.OR,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为字符串起始匹配的消息事件处理器。
        必须首先含有以 target 起始的文本，才能被进一步处理
        """
        start_matcher = StartMatch(target, logic_mode)

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=MsgEventHandler,
                params=[
                    start_matcher,
                    None,
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_contain_match(
        cls,
        target: str | list[str],
        logic_mode: LogicMode = LogicMode.OR,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为字符串包含匹配的消息事件处理器。
        文本必须首先包含 target，才能被进一步处理
        """
        contain_matcher = ContainMatch(target, logic_mode)

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=MsgEventHandler,
                params=[
                    contain_matcher,
                    None,
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_full_match(
        cls,
        target: str | list[str],
        logic_mode: LogicMode = LogicMode.OR,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为字符串相等匹配的消息事件处理器。
        文本必须首先与 target 内容完全一致，才能被进一步处理
        """
        full_matcher = FullMatch(target, logic_mode)

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=MsgEventHandler,
                params=[
                    full_matcher,
                    None,
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_end_match(
        cls,
        target: str | list[str],
        logic_mode: LogicMode = LogicMode.OR,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为字符串结尾匹配的消息事件处理器。
        文本必须首先以 target 结尾，才能被进一步处理
        """
        end_matcher = EndMatch(target, logic_mode)

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=MsgEventHandler,
                params=[
                    end_matcher,
                    None,
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_regex_match(
        cls,
        target: str,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为字符串正则匹配的消息事件处理器。
        文本必须包含指定的正则内容，才能被进一步处理
        """
        regex_matcher = RegexMatch(target)

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=MsgEventHandler,
                params=[
                    regex_matcher,
                    None,
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_request(
        cls,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为请求事件处理器
        """

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=ReqEventHandler,
                params=[
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_friend_request(
        cls,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为私聊请求事件处理器
        """
        friend_checker = FriendReqChecker()
        if checker is not None:
            wrapped_checker = friend_checker & checker
        else:
            wrapped_checker = friend_checker

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=ReqEventHandler,
                params=[
                    wrapped_checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_group_request(
        cls,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为群请求事件处理器
        """
        group_checker = GroupReqChecker()
        if checker is not None:
            wrapped_checker = group_checker & checker
        else:
            wrapped_checker = group_checker

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=ReqEventHandler,
                params=[
                    wrapped_checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_notice(
        cls,
        type: str = Literal[
            "group_upload",
            "group_admin",
            "group_decrease",
            "group_increase",
            "group_ban",
            "friend_add",
            "group_recall",
            "friend_recall",
            "group_card",
            "offline_file",
            "client_status",
            "essence",
            "notify",
            "honor",
            "poke",
            "lucky_king",
            "title",
            "ALL",
        ],
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为通知事件处理器
        """
        type_checker = NoticeTypeChecker(type)
        if checker is not None:
            wrapped_checker = type_checker & checker
        else:
            wrapped_checker = type_checker

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=NoticeEventHandler,
                params=[
                    wrapped_checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args

    @classmethod
    def on_meta_event(
        cls,
        checker: "BotChecker" = None,
        priority: PriorLevel = PriorLevel.MEAN,
        timeout: int = None,
        block: bool = False,
        temp: bool = False,
        session_rule: "SessionRule" = None,
        session_hold: bool = False,
        direct_rouse: bool = False,
        conflict_wait: bool = False,
        conflict_cb: Callable[[], Coroutine[Any, Any, None]] = None,
        overtime_cb: Callable[[], Coroutine[Any, Any, None]] = None,
    ) -> Callable:
        """
        使用该装饰器，将方法标记为元事件处理器
        """

        def make_args(
            executor: Callable[[], Coroutine[Any, Any, None]]
        ) -> EventHandlerArgs:
            return EventHandlerArgs(
                executor=executor,
                type=MetaEventHandler,
                params=[
                    checker,
                    priority,
                    timeout,
                    block,
                    temp,
                    session_rule,
                    session_hold,
                    direct_rouse,
                    conflict_wait,
                    conflict_cb,
                    overtime_cb,
                ],
            )

        return make_args



import asyncio as aio
import traceback

from ..types.abc import AbstractDispatcher
from ..types.exceptions import *
from ..types.typing import *

if TYPE_CHECKING:
    from ..plugin.bot import BotHookBus
    from ..plugin.handler import EventHandler
    from ..types.abc import BotEvent
    from ..utils.logger import Logger


class BotDispatcher(AbstractDispatcher):
    """
    bot 分发模块。负责将传递的事件分发到各事件通道
    （接收的事件类型：消息、请求、通知和元事件）
    """

    def __init__(
        self,
        channel_map: dict[str, tuple[Type["EventHandler"], ...]],
        bot_bus: Type["BotHookBus"],
        logger: "Logger",
    ) -> None:
        super().__init__()
        self.handlers: Dict[Type["EventHandler"], List["EventHandler"]] = {}

        self.logger = logger
        self._channel_map = channel_map
        self._bot_bus = bot_bus
        self._ready_signal = aio.Event()

        for tuple in self._channel_map.values():
            for channel in tuple:
                if channel not in self.handlers.keys():
                    self.handlers[channel] = []

    def add_handlers(self, all_handlers: List["EventHandler"]) -> None:
        """
        绑定事件处理器列表
        """
        for handler in all_handlers:
            self.handlers[handler.__class__].append(handler)
        for k in self.handlers.keys():
            self.handlers[k] = sorted(
                self.handlers[k], key=lambda x: x.priority, reverse=True
            )
        self._ready_signal.set()

    async def broadcast(self, event: "BotEvent", channel: Type["EventHandler"]) -> None:
        """
        向指定的通道推送事件
        """
        permit_priority = PriorLevel.MIN.value
        handlers = self.handlers[channel]
        for handler in handlers:
            # 事件处理器优先级不够，则不分配给它处理
            if handler.priority < permit_priority:
                continue
            # evoke 返回的值用于判断，事件处理器内部经过各种检查后，是否选择处理这个事件。
            if not (await handler.evoke(event)):
                # 如果决定不处理，则会跳过此次循环（也就是不进行“可能存在的优先级阻断操作”）
                continue
            if handler.set_block and handler.priority > permit_priority:
                permit_priority = handler.priority

    async def dispatch(self, event: "BotEvent") -> None:
        """
        把事件分发到对应的事件通道
        """
        await self._ready_signal.wait()
        await self._bot_bus.emit(BotLife.EVENT_BUILT, event, wait=True)
        try:
            for channel in self._channel_map[event.type]:
                await self.broadcast(event, channel)
        except Exception as e:
            self.logger.error(f"bot dispatcher 抛出异常：[{e.__class__.__name__}] {e}")
            self.logger.debug(f"异常点的事件记录为：{event.raw}")
            self.logger.debug("异常回溯栈：\n" + traceback.format_exc().strip("\n"))



import os
import sys
from typing import Any, Dict


class MetaInfo:
    def __init__(self) -> None:
        self.VER = "2.3.2"
        self.PROJ_NAME = "MeloBot"
        self.PROJ_DESC = "A qbot module with friendly interface, session control and plugin-supported."
        self.PROJ_SRC = "https://github.com/aicorein/melobot"
        self.AUTHOR = "aicorein"
        self.AUTHOR_EMAIL = "melodyecho@glowmem.com"
        self.ARGV = sys.argv
        self.PLATFORM = sys.platform
        self.PY_VER = sys.version
        self.PY_INFO = sys.version_info
        self.OS_SEP = os.sep
        self.PATH_SEP = os.pathsep
        self.LINE_SEP = os.linesep
        self.ENV = os.environ

    def get_all(self) -> Dict[str, Any]:
        return dict(self.__dict__.items())


MODULE_MODE_FLAG = "MELOBOT_MODULE"
MODULE_MODE_SET = "1"
MODULE_MODE_UNSET = "0"
EXIT_CLOSE = 0x0
EXIT_ERROR = 0x1
EXIT_RESTART = 0x10



import asyncio as aio
import traceback

from ..types.exceptions import *
from ..types.typing import *

if TYPE_CHECKING:
    from ..models.event import MessageEvent, ResponseEvent


class FormatInfo:
    """
    格式化信息对象
    """

    def __init__(
        self,
        src: str,
        src_desc: Optional[str],
        src_expect: Optional[str],
        idx: int,
        exc_type: Exception,
        exc_tb: ModuleType,
        cmd_name: str,
        event: "MessageEvent",
    ) -> None:
        self.src = src
        self.src_desc = src_desc
        self.src_expect = src_expect
        self.idx = idx
        self.exc_type = exc_type
        self.exc_tb = exc_tb
        self.cmd_name = cmd_name
        self.event = event


class ArgFormatter:
    """
    参数格式化器。将参数格式化为对应类型的对象。格式化后可添加校验，也可自定义失败提示信息。
    还可设置默认值。不提供默认值参数时为 Void，代表不使用默认值
    """

    def __init__(
        self,
        convert: Callable[[str], Any] = None,
        verify: Callable[[Any], bool] = None,
        src_desc: str = None,
        src_expect: str = None,
        default: Any = Void,
        default_replace_flag: str = None,
        convert_fail: Callable[[FormatInfo], Coroutine[Any, Any, None]] = None,
        verify_fail: Callable[[FormatInfo], Coroutine[Any, Any, None]] = None,
        arglack: Callable[[FormatInfo], Coroutine[Any, Any, None]] = None,
    ) -> None:
        self.convert = convert
        self.verify = verify
        self.src_desc = src_desc
        self.src_expect = src_expect
        self.default = default
        self.default_replace_flag = default_replace_flag
        if self.default is Void and self.default_replace_flag is not None:
            raise ArgFormatInitError(
                "初始化参数格式化器时，使用“默认值替换标记”必须同时设置默认值"
            )

        self.convert_fail = convert_fail
        self.verify_fail = verify_fail
        self.arg_lack = arglack

    def _get_val(self, args: ParseArgs, idx: int) -> Any:
        if self.default is Void:
            if args.vals is None or len(args.vals) < idx + 1:
                raise ArgLackError
            else:
                return args.vals[idx]
        if args.vals is None:
            args.vals = [self.default]
        elif len(args.vals) < idx + 1:
            args.vals.append(self.default)
        return args.vals[idx]

    def format(
        self,
        cutsom_msg_func: Callable[..., Coroutine[Any, Any, "ResponseEvent"]],
        cmd_name: str,
        event: "MessageEvent",
        args: ParseArgs,
        idx: int,
    ) -> bool:
        """
        格式化参数为对应类型的变量
        """
        try:
            src = self._get_val(args, idx)
            if (
                self.default_replace_flag is not None
                and src == self.default_replace_flag
            ):
                src = self.default
            res = self.convert(src) if self.convert is not None else src

            if self.verify is None:
                pass
            elif self.verify(res):
                pass
            else:
                raise ArgVerifyFailed
            args.vals[idx] = res
            return True
        except ArgVerifyFailed as e:
            info = FormatInfo(
                src, self.src_desc, self.src_expect, idx, e, traceback, cmd_name, event
            )
            if self.verify_fail:
                aio.create_task(self.verify_fail(info))
            else:
                self._verify_fail_default(info, cutsom_msg_func)
            return False
        except ArgLackError as e:
            info = FormatInfo(
                None, self.src_desc, self.src_expect, idx, e, traceback, cmd_name, event
            )
            if self.arg_lack:
                aio.create_task(self.arg_lack(info))
            else:
                self._arglack_default(info, cutsom_msg_func)
            return False
        except Exception as e:
            info = FormatInfo(
                src, self.src_desc, self.src_expect, idx, e, traceback, cmd_name, event
            )
            if self.convert_fail:
                aio.create_task(self.convert_fail(info))
            else:
                self._convert_fail_default(info, cutsom_msg_func)
            return False

    def _convert_fail_default(
        self,
        info: FormatInfo,
        msg_func: Callable[..., Coroutine[Any, Any, "ResponseEvent"]],
    ) -> str:
        e_class = info.exc_type.__class__.__name__
        src = info.src.__repr__() if isinstance(info.src, str) else info.src
        tip = f"第 {info.idx+1} 个参数"
        tip += (
            f"（{info.src_desc}）无法处理，给定的值为：{src}。"
            if info.src_desc
            else f"给定的值 {src} 无法处理。"
        )
        tip += f"参数要求：{info.src_expect}。" if info.src_expect else ""
        tip += f"\n详细错误描述：[{e_class}] {info.exc_type}"
        tip = tip + f"命令 {info.cmd_name} 参数格式化失败：\n"
        aio.create_task(
            msg_func(
                tip, info.event.is_private(), info.event.sender.id, info.event.group_id
            )
        )

    def _verify_fail_default(
        self,
        info: FormatInfo,
        msg_func: Callable[..., Coroutine[Any, Any, "ResponseEvent"]],
    ) -> None:
        src = info.src.__repr__() if isinstance(info.src, str) else info.src
        tip = f"第 {info.idx+1} 个参数"
        tip += (
            f"（{info.src_desc}）不符合要求，给定的值为：{src}。"
            if info.src_desc
            else f"给定的值 {src} 不符合要求。"
        )
        tip += f"参数要求：{info.src_expect}。" if info.src_expect else ""
        tip = tip + f"命令 {info.cmd_name} 参数格式化失败：\n"
        aio.create_task(
            msg_func(
                tip, info.event.is_private(), info.event.sender.id, info.event.group_id
            )
        )

    def _arglack_default(
        self,
        info: FormatInfo,
        msg_func: Callable[..., Coroutine[Any, Any, "ResponseEvent"]],
    ) -> None:
        tip = f"第 {info.idx+1} 个参数"
        tip += f"（{info.src_desc}）缺失。" if info.src_desc else f"缺失。"
        tip += f"参数要求：{info.src_expect}。" if info.src_expect else ""
        tip = tip + f"命令 {info.cmd_name} 参数格式化失败：\n"
        aio.create_task(
            msg_func(
                tip, info.event.is_private(), info.event.sender.id, info.event.group_id
            )
        )



import asyncio as aio
import inspect
import os
import sys
import traceback

import websockets.exceptions as wse

from .context.session import BotSessionManager
from .controller.dispatcher import BotDispatcher
from .controller.responder import BotResponder
from .io.linker import BotLinker
from .meta import EXIT_CLOSE, EXIT_RESTART, MODULE_MODE_FLAG, MODULE_MODE_SET, MetaInfo
from .models.event import BotEventBuilder
from .plugin.bot import BOT_PROXY, BotHookBus
from .plugin.handler import EVENT_HANDLER_MAP
from .plugin.ipc import PluginBus, PluginStore
from .plugin.plugin import Plugin, PluginLoader
from .types.exceptions import *
from .types.typing import *
from .utils.config import BotConfig
from .utils.logger import get_logger

if TYPE_CHECKING:
    from .utils.logger import Logger

if sys.platform != "win32":
    import uvloop

    aio.set_event_loop_policy(uvloop.EventLoopPolicy())


class MeloBot:
    """
    bot 模块。负责整个 bot 的启动、加载和管理。
    指定 restart_tip 可在重启时自动回复
    """

    def __init__(self) -> None:
        self.config: BotConfig
        self.info = MetaInfo()
        # 不要更改这个属性名
        self._logger: "Logger" = None

        self.life: aio.Task = None
        self.plugins: Dict[str, Plugin] = {}
        self.loader = PluginLoader

        self.linker: BotLinker
        self.responder: BotResponder
        self.dispatcher: BotDispatcher
        self.ctx_manager = BotSessionManager
        self.event_builder = BotEventBuilder
        self.plugin_bus = PluginBus
        self.plugin_store = PluginStore
        self.bot_bus = BotHookBus
        self.proxy = BOT_PROXY

        self.__init_flag__ = False
        self.exit_code = EXIT_CLOSE

    @property
    def loop(self) -> aio.AbstractEventLoop:
        return aio.get_running_loop()

    @property
    def slack(self) -> bool:
        return self.linker.slack

    @slack.setter
    def slack(self, value: bool) -> None:
        self.linker.slack = value

    @property
    def logger(self) -> "Logger":
        return self._logger if self._logger else get_logger()

    def init(self, config_dir) -> None:
        """
        为 bot 核心加载配置文件
        """
        if self.__init_flag__:
            self.logger.error("bot 不能重复初始化")
            exit(self.exit_code)

        self.config = BotConfig(config_dir)
        self._logger = get_logger(self.config.log_dir_path, self.config.log_level)
        self.linker = BotLinker(
            self.config.connect_host,
            self.config.connect_port,
            self.config.max_conn_try,
            self.config.conn_try_interval,
            self.config.cooldown_time,
            self.event_builder,
            self.bot_bus,
            self.logger,
        )
        self.responder = BotResponder(self.logger)
        self.dispatcher = BotDispatcher(EVENT_HANDLER_MAP, self.bot_bus, self.logger)
        self.ctx_manager._bind(self.responder)
        self.plugin_bus._bind(self.logger)
        self.bot_bus._bind(self.logger)

        self.logger.info("欢迎使用 melobot v2")
        self.logger.info(
            f"melobot 在 AGPL3 协议下开源发行。更多请参阅：{self.info.PROJ_SRC}"
        )
        self.logger.info(f"运行版本：{self.info.VER}，平台：{self.info.PLATFORM}")
        if os.environ.get(MODULE_MODE_FLAG) == MODULE_MODE_SET:
            self.logger.info("当前运行模式：模块运行模式")
        else:
            self.logger.info("当前运行模式：脚本运行模式")
        self.logger.info("bot 核心初始化完成")
        self.__init_flag__ = True

    def load_plugin(self, plugin_target: str | Type[Plugin]) -> None:
        """
        为 bot 加载运行插件。支持传入插件起始目录字符串（绝对路径）或插件类
        """
        if not self.__init_flag__:
            self.logger.error("加载插件必须在初始化之后进行")
            exit(self.exit_code)

        plugin_dir = (
            inspect.getfile(plugin_target)
            if not isinstance(plugin_target, str)
            else plugin_target
        )
        self.logger.debug(f"尝试加载来自 {plugin_dir} 的插件")
        plugin = self.loader.load(plugin_target, self.logger, self.responder)
        exist_plugin = self.plugins.get(plugin.ID)
        if exist_plugin is None:
            self.plugins[plugin.ID] = plugin
            self.dispatcher.add_handlers(plugin._handlers)
            self.logger.info(f"成功加载插件：{plugin.ID}")
        else:
            self.logger.error(
                f"加载插件出错：插件名称重复, 尝试加载：{plugin_dir}，已加载：{exist_plugin.plugin_dir}"
            )
            exit(self.exit_code)

    def load_plugins(self, plugins_dir: str) -> None:
        """
        从插件目录加载多个 bot 运行插件。但必须遵循标准插件格式
        """
        items = os.listdir(plugins_dir)
        for item in items:
            path = os.path.join(plugins_dir, item)
            if os.path.isdir(path) and os.path.basename(path) != "__pycache__":
                self.load_plugin(path)

    async def _run(self) -> None:
        if len(self.plugins) == 0:
            self.logger.warning("没有加载任何插件，bot 将不会有任何操作")

        await self.bot_bus.emit(BotLife.LOADED)
        self.responder.bind(self.linker)
        self.linker.bind(self.dispatcher, self.responder)
        try:
            async with self.linker:
                self.life = aio.create_task(self.linker.listen())
                self.proxy._bind(self)
                self.logger.info("bot 开始正常运行")
                await self.life
        except wse.ConnectionClosed:
            pass
        except Exception as e:
            self.logger.error(f"bot 核心无法继续运行。异常：{e}")
            self.logger.debug("异常回溯栈：\n" + traceback.format_exc().strip("\n"))
        finally:
            await self.bot_bus.emit(BotLife.BEFORE_STOP, wait=True)
            self.logger.info("bot 已清理运行时资源并关闭")
            exit(self.exit_code)

    def run(self) -> None:
        """
        运行 bot
        """
        if not self.__init_flag__:
            self.logger.error("必须先初始化才能启动")
            exit(self.exit_code)
        try:
            """
            一定要手动设置事件循环。在启动前，部分核心模块初始化异步对象，
            已经生成了事件循环，不设置而直接使用 asyncio.run 将会运行一个新的事件循环，
            不同事件循环的异步对象无法直接通信
            """
            aio.set_event_loop(aio.get_event_loop())
            # 使用 asyncio.run 可以保证发生各种异常时一定取消所有任务（包括键盘中断）
            aio.run(self._run())
        except KeyboardInterrupt:
            pass

    async def close(self) -> None:
        """
        停止 bot
        """
        if self.life is None:
            self.logger.error("bot 尚未运行，无需停止")
            exit(self.exit_code)
        await self.bot_bus.emit(BotLife.BEFORE_CLOSE, wait=True)
        self.life.cancel()

    async def restart(self) -> None:
        """
        重启 bot
        """
        if self.life is None:
            self.logger.error("bot 尚未运行，无需重启")
            exit(self.exit_code)
        if os.environ.get(MODULE_MODE_FLAG) != MODULE_MODE_SET:
            self.logger.error("只有在模块运行模式下，才能使用 bot 重启功能")
            return
        self.logger.info("bot 即将进行重新启动...")
        self.exit_code = EXIT_RESTART
        self.life.cancel()



import asyncio as aio
import time
from contextvars import ContextVar, Token
from copy import deepcopy
from functools import wraps

from ..types.abc import SessionRule
from ..types.exceptions import *
from ..types.typing import *
from ..utils.atools import get_twin_event

if TYPE_CHECKING:
    from ..models.event import MessageEvent, MetaEvent, NoticeEvent, RequestEvent
    from ..types.abc import AbstractResponder, BotAction, BotEvent
    from ..plugin.handler import EventHandler


class BotSession:
    """
    Bot Session 类。不需要直接实例化，必须通过 BotSessionBuilder 构造。
    """

    def __init__(
        self,
        manager: Type["BotSessionManager"],
        responder: "AbstractResponder",
        space_tag: "EventHandler" = None,
    ) -> None:
        super().__init__()
        self.store = {}
        self.timestamp = time.time()
        self.hup_times: List[float] = []
        self.events: List[
            "MessageEvent" | "RequestEvent" | "MetaEvent" | "NoticeEvent"
        ] = []
        self.actions: List["BotAction"] = []

        self._manager = manager
        self._responder = responder
        # session 是否空闲的标志，由 BotSessionManager 修改和管理
        self._free_signal = aio.Event()
        self._free_signal.set()
        # session 是否挂起的标志。二者互为孪生反状态。由 BotSessionManager 修改和管理
        # 注意 session 挂起时一定是非空闲和非过期的
        self._hup_signal, self._awake_signal = get_twin_event()
        # session 是否过期的标志，由 BotSessionManager 修改和管理
        self._expired = False
        # 用于标记该 session 属于哪个 session 空间，如果为 None 则表明是空 session 或是一次性 session
        # 其实这里如果传入 space_tag 则一定是所属 handler 的引用
        self._space_tag: Optional["EventHandler"] = space_tag
        # 所属 handler 的引用（和 space_tag 不一样，所有在 handler 中产生的 session，此属性必为非空）
        self._handler: "EventHandler" = None

    @property
    def event(
        self,
    ) -> Union["MessageEvent", "RequestEvent", "MetaEvent", "NoticeEvent"]:
        try:
            return next(reversed(self.events))
        except StopIteration:
            raise BotSessionError("当前 session 下不存在可用的 event")

    @property
    def last_action(self) -> "BotAction":
        try:
            return next(reversed(self.actions))
        except StopIteration:
            raise BotSessionError("当前 session 下尚未进行过 action 操作")

    @property
    def last_hup(self) -> Optional[float]:
        try:
            return next(reversed(self.hup_times))
        except StopIteration:
            return None

    @property
    def args(self):
        if self._handler is not None and self.event._args_map is not None:
            args_group = self.event._args_map.get(self._handler.parser.id)
            if self._handler.parser.target is None:
                return deepcopy(args_group)
            for cmd_name in self._handler.parser.target:
                args = args_group.get(cmd_name)
                if args is not None:
                    return deepcopy(args.vals)
            raise BotSessionError("尝试获取的命令解析参数不存在")
        return None

    async def hup(self, overtime: int = None) -> None:
        """
        当前 session 挂起（也就是所在方法的挂起）。直到满足同一 session_rule 的事件重新进入，
        session 所在方法便会被唤醒。但如果设置了超时时间且在唤醒前超时，则会强制唤醒 session，
        并抛出 SessionHupTimeout 异常
        """
        self._manager._hup(self)
        if overtime is None:
            await self._awake_signal.wait()
        elif overtime <= 0:
            raise BotSessionError("session 挂起超时时间（秒数）必须 > 0")
        else:
            await aio.wait(
                [self._awake_signal.wait(), aio.sleep(overtime)],
                return_when=aio.FIRST_COMPLETED,
            )
            if self._awake_signal.is_set():
                return
            self._manager._rouse(self)
            raise SessionHupTimeout("session 挂起超时")

    def destory(self) -> None:
        """
        销毁方法。
        其他 session 调用会立即清空 session 存储、事件记录、挂起时间记录。
        如果调用 session 有 space_tag，还会从存储空间中移除该 session
        """
        if self.event is None:
            pass
        else:
            self._manager._expire(self)

    def store_get(self, key: Any) -> Any:
        return self.store[key]

    def store_add(self, key: Any, val: Any) -> None:
        self.store[key] = val

    def store_update(self, store: dict) -> None:
        self.store.update(store)

    def store_remove(self, key: Any) -> None:
        self.store.pop(key)

    def store_clear(self) -> None:
        self.store.clear()


class SessionLocal:
    """
    session 自动上下文
    """

    __slots__ = tuple(
        list(filter(lambda x: not (len(x) >= 2 and x[:2] == "__"), dir(BotSession)))
        + ["__storage__"]
    )

    def __init__(self) -> None:
        object.__setattr__(self, "__storage__", ContextVar("session_ctx"))
        self.__storage__: ContextVar[BotSession]

    def __setattr__(self, __name: str, __value: Any) -> None:
        setattr(self.__storage__.get(), __name, __value)

    def __getattr__(self, __name: str) -> Any:
        return getattr(self.__storage__.get(), __name)

    def _add_ctx(self, ctx: BotSession) -> Token:
        return self.__storage__.set(ctx)

    def _del_ctx(self, token: Token) -> None:
        self.__storage__.reset(token)


SESSION_LOCAL = SessionLocal()
SESSION_LOCAL: BotSession


class AttrSessionRule(SessionRule):
    def __init__(self, *attrs) -> None:
        super().__init__()
        self.attrs = attrs

    def _get_val(self, e: "BotEvent", attrs: Tuple[str, ...]) -> Any:
        val = e
        try:
            for attr in attrs:
                val = getattr(val, attr)
        except AttributeError:
            raise BotSessionError(f"session 规则指定的属性 {attr} 不存在")
        return val

    def compare(self, e1: "BotEvent", e2: "BotEvent") -> bool:
        return self._get_val(e1, self.attrs) == self._get_val(e2, self.attrs)


class BotSessionManager:
    STORAGE: Dict["EventHandler", Set[BotSession]] = {}
    HUP_STORAGE: Dict["EventHandler", Set[BotSession]] = {}
    # 各个 handler 对饮的操作锁
    WORK_LOCKS: Dict["EventHandler", aio.Lock] = {}
    # 用来标记 cls.get 等待一个挂起的 session 时的死锁
    DEADLOCK_FLAGS: Dict["EventHandler", aio.Event] = {}
    # 对应每个 handler 的 try_attach 过程的操作锁
    ATTACH_LOCKS: Dict["EventHandler", aio.Lock] = {}
    RESPONDER: "AbstractResponder"

    @classmethod
    def _bind(cls, responder: "AbstractResponder") -> None:
        cls.RESPONDER = responder

    @classmethod
    def register(cls, handler: "EventHandler") -> None:
        """
        以 handler 为键，注册 handler 对应的 session 空间、操作锁和挂起 session 空间
        """
        cls.STORAGE[handler] = set()
        cls.WORK_LOCKS[handler] = aio.Lock()
        cls.HUP_STORAGE[handler] = set()
        cls.DEADLOCK_FLAGS[handler] = aio.Event()
        cls.ATTACH_LOCKS[handler] = aio.Lock()

    @classmethod
    def inject(cls, session: BotSession, handler: "EventHandler") -> None:
        """
        handler 内绑定 handler 引用到 session
        """
        session._handler = handler

    @classmethod
    def _attach(cls, event: "BotEvent", handler: "EventHandler") -> bool:
        """
        session 附着操作，临界区操作。只能在 cls.try_attach 中进行
        """
        session = None
        for s in cls.HUP_STORAGE[handler]:
            # session 的挂起方法，保证 session 一定未过期，因此不进行过期检查
            if handler._rule.compare(s.event, event):
                session = s
                break
        # 如果获得一个挂起的 session，它一定是可附着的，附着后需要唤醒
        if session:
            session.events.append(event)
            cls._rouse(session)
            return True
        return False

    @classmethod
    async def try_attach(cls, event: "BotEvent", handler: "EventHandler") -> bool:
        """
        检查是否有挂起的 session 可供 event 附着。
        如果有则附着并唤醒，并返回 True。否则返回 False。
        """
        if handler._rule is None:
            return False

        async with cls.ATTACH_LOCKS[handler]:
            t1 = aio.create_task(cls.DEADLOCK_FLAGS[handler].wait(), name="flag")
            t2 = aio.create_task(cls.WORK_LOCKS[handler].acquire(), name="lock")
            done, _ = await aio.wait([t1, t2], return_when=aio.FIRST_COMPLETED)
            # 等待完成后，一定要记得取消另一个任务！否则可能异常加锁
            if done.pop().get_name() == "flag":
                res = cls._attach(event, handler)
                cls.DEADLOCK_FLAGS[handler].clear()
                t2.cancel()
                return res
            else:
                res = cls._attach(event, handler)
                cls.WORK_LOCKS[handler].release()
                t1.cancel()
                return res

    @classmethod
    def _hup(cls, session: BotSession) -> None:
        """
        挂起 session
        """
        if session._space_tag is None:
            raise BotSessionError(
                "当前 session 上下文因为缺乏 session_rule 作为唤醒标志，无法挂起"
            )
        elif session._expired:
            raise BotSessionError("过期的 session 不能被挂起")
        session.hup_times.append(time.time())
        cls.STORAGE[session._space_tag].remove(session)
        cls.HUP_STORAGE[session._space_tag].add(session)
        session._awake_signal.clear()

    @classmethod
    def _rouse(cls, session: BotSession) -> None:
        """
        唤醒 session
        """
        cls.HUP_STORAGE[session._space_tag].remove(session)
        cls.STORAGE[session._space_tag].add(session)
        session._awake_signal.set()

    @classmethod
    def _expire(cls, session: BotSession) -> None:
        """
        标记该 session 为过期状态，并进行销毁操作（如果存在于某个 session_space，则从中移除）
        """
        if session._expired:
            return
        session.events.clear()
        session.hup_times.clear()
        session.store_clear()
        session._expired = True
        if session._space_tag:
            cls.STORAGE[session._space_tag].remove(session)

    @classmethod
    def recycle(cls, session: BotSession, alive: bool = False) -> None:
        """
        session 所在方法运行结束后，回收 session。
        默认将 session 销毁。若 alive 为 True，则保留
        """
        session._free_signal.set()
        if not alive:
            cls._expire(session)

    @classmethod
    async def get(cls, event: "BotEvent", handler: "EventHandler") -> Optional[BotSession]:
        """
        handler 内获取 session 方法。自动根据 handler._rule 判断是否需要映射到 session_space 进行存储。
        然后根据具体情况，获取已有 session 或新建 session。当尝试获取非空闲 session 时，如果 handler 指定不等待则返回 None
        """
        if handler._rule:
            # session_space, session._free_signal 竞争，需要加锁
            async with cls.WORK_LOCKS[handler]:
                session = await cls._get_on_rule(event, handler)
                # 必须在锁的保护下修改 session._free_signal
                if session:
                    session._free_signal.clear()
        else:
            session = cls._make(event, handler)
            session._free_signal.clear()

        return session

    @classmethod
    def _make(cls, event: "BotEvent", handler: "EventHandler" = None) -> BotSession:
        """
        内部使用的创建 session 方法。如果 handler 为空，即缺乏 space_tag，则为一次性 session。
        或 handler._rule 为空，则也为一次性 session
        """
        if handler:
            if handler._rule:
                session = BotSession(cls, cls.RESPONDER, handler)
                session.events.append(event)
                cls.STORAGE[handler].add(session)
                return session
        session = BotSession(cls, cls.RESPONDER)
        session.events.append(event)
        return session

    @classmethod
    def make_empty(cls) -> BotSession:
        """
        创建空 session。即不含 event 和 space_tag 标记的 session
        """
        return BotSession(cls, cls.RESPONDER)

    @classmethod
    def make_temp(cls, event: "BotEvent") -> BotSession:
        """
        创建一次性 session。确定无需 session 管理机制时可以使用。
        否则一定使用 cls.get 方法
        """
        return cls._make(event)

    @classmethod
    async def _get_on_rule(
        cls, event: "BotEvent", handler: "EventHandler"
    ) -> Optional[BotSession]:
        """
        根据 handler 具体情况，从对应 session_space 中获取 session 或新建 session。
        或从 hup_session_space 中唤醒 session，或返回 None
        """
        session = None
        check_rule, session_space, hup_session_space, conflict_wait = (
            handler._rule,
            cls.STORAGE[handler],
            cls.HUP_STORAGE[handler],
            handler._wait_flag,
        )

        # for 循环都需要即时 break，保证遍历 session_space 时没有协程切换。因为切换后 session_space 可能发生变动
        for s in session_space:
            if check_rule.compare(s.event, event) and not s._expired:
                session = s
                break
        # 如果会话不存在，生成一个新 session 变量
        if session is None:
            return cls._make(event, handler)
        # 如果会话存在，且未过期，且空闲，则附着到这个 session 上
        if session._free_signal.is_set():
            session.events.append(event)
            return session
        # 如果会话存在，且未过期，但是不空闲，选择不等待
        if not conflict_wait:
            return None
        # 如果会话存在，且未过期，但是不空闲，选择等待，此时就不得不陷入等待（即将发生协程切换）
        await aio.wait(
            [session._free_signal.wait(), session._hup_signal.wait()],
            return_when=aio.FIRST_COMPLETED,
        )
        if session._hup_signal.is_set():
            cls.DEADLOCK_FLAGS[handler].set()
            await session._free_signal.wait()
        """
        重新切换回本协程后，session 有可能变为过期，但此时一定是空闲的。
        同时一定是非挂起状态。因为上面解决了可能存在的挂起死锁问题。
        即使该 session 因过期被所在的 session_space 清除也无妨，因为此处有引用，
        该 session 并不会消失。且此处不操作 session_space，无需担心 session_space 变动
        """
        # 如果过期，生成一个新的 session 变量
        if session._expired:
            return cls._make(event, handler)
        # 如果未过期，则附着到这个 session 上
        else:
            session.events.append(event)
            return session

    @classmethod
    def _activate(cls, action_getter):
        """
        对 action 构造器进行装饰，使产生的 action “活化”。
        让其能自动识别上下文，自动附加触发 event，并自动完成发送过程
        """
        try:
            if SESSION_LOCAL._expired:
                raise BotSessionError(
                    "当前  session 上下文已有过期标记，无法再执行 action 操作"
                )
        except LookupError:
            pass

        @wraps(action_getter)
        async def activated_action(*args, **kwargs):
            action: "BotAction" = await action_getter(*args, **kwargs)
            try:
                if len(SESSION_LOCAL.events) > 0:
                    action._fill_trigger(SESSION_LOCAL.event)
            except LookupError:
                pass

            if not action.ready:
                return action
            try:
                SESSION_LOCAL.actions.append(action)
            except LookupError:
                pass
            if action.resp_id is None:
                return await cls.RESPONDER.take_action(action)
            else:
                return await (await cls.RESPONDER.take_action_wait(action))

        return activated_action


def any_event():
    """
    获取当前 session 上下文下标注为联合类型的 event
    """
    return SESSION_LOCAL.event


def msg_event() -> "MessageEvent":
    """
    获取当前 session 上下文下的 MessageEvent
    """
    return SESSION_LOCAL.event


def notice_event() -> "NoticeEvent":
    """
    获取当前 session 上下文下的 NoticeEvent
    """
    return SESSION_LOCAL.event


def req_evnt() -> "RequestEvent":
    """
    获取当前 session 上下文下的 RequestEvent
    """
    return SESSION_LOCAL.event


def meta_event() -> "MetaEvent":
    """
    获取当前 session 上下文下的 MetaEvent
    """
    return SESSION_LOCAL.event


def msg_text():
    """
    获取当前 session 上下文下的消息的纯文本数据
    """
    return SESSION_LOCAL.event.text


def msg_args():
    """
    获取当前 session 上下文下的解析参数，如果不存在则返回 None
    """
    return SESSION_LOCAL.args


async def pause(overtime: int = None) -> None:
    """
    当前 session 挂起（也就是所在方法的挂起）。直到满足同一 session_rule 的事件重新进入，
    session 所在方法便会被唤醒。但如果设置了超时时间且在唤醒前超时，则会强制唤醒 session，
    并抛出 SessionHupTimeout 异常
    """
    await SESSION_LOCAL.hup(overtime)



import re

from ..types.abc import BotEvent, BotParser
from ..types.exceptions import *
from ..types.typing import *

if TYPE_CHECKING:
    from ..models.event import ResponseEvent
    from .formatter import ArgFormatter


class CmdParser(BotParser):
    """
    命令解析器，通过解析命令名和参数的形式，解析字符串。
    命令起始符和命令间隔符不允许包含 引号，各种括号，反斜杠，数字，英文，回车符，换行符，制表符
    """

    def __init__(
        self,
        cmd_start: str | list[str],
        cmd_sep: str | list[str],
        target: str | list[str] = None,
        formatters: List[Optional["ArgFormatter"]] = None,
    ) -> None:
        i1 = cmd_start if isinstance(cmd_start, str) else "".join(cmd_start)
        i2 = cmd_sep if isinstance(cmd_sep, str) else "".join(cmd_sep)
        super().__init__(i1 + "\u0000" + i2)
        self.target = target if isinstance(target, list) or target is None else [target]
        self.formatters = [] if formatters is None else formatters
        self.need_format = (
            True if target is not None and len(self.formatters) > 0 else False
        )
        if target is None and len(self.formatters) > 0:
            raise ArgParseError("不指定 target 来匹配具体的命令名时，无法进行格式化")

        self.start_tokens = cmd_start if isinstance(cmd_start, list) else [cmd_start]
        self.sep_tokens = cmd_sep if isinstance(cmd_sep, list) else [cmd_sep]
        self.ban_regex = re.compile(r"[\'\"\\\(\)\[\]\{\}\r\n\ta-zA-Z0-9]")
        self._build_parse_regex()

        if self.ban_regex.findall("".join(cmd_start) + "".join(cmd_sep)):
            raise ArgParseError("存在命令解析器不支持的命令起始符，或命令间隔符")

    def _build_parse_regex(self):
        """
        建立用于命令解析的正则 Pattern 对象，包含命令起始符正则 pattern 和
        命令间隔符正则 pattern
        """
        temp_regex = re.compile(
            r"([\`\-\=\~\!\@\#\$\%\^\&\*\(\)\_\+\[\]\{\}\|\:\,\.\/\<\>\?])"
        )
        if not len(set(self.sep_tokens) & set(self.start_tokens)):
            self.cmd_sep = [
                temp_regex.sub(r"\\\1", sep_token) for sep_token in self.sep_tokens
            ]
            self.cmd_start = [
                temp_regex.sub(r"\\\1", start_token)
                for start_token in self.start_tokens
            ]
            self.sep_parse_regex = re.compile(rf"{'|'.join(self.cmd_sep)}")
            self.start_parse_regex = re.compile(rf"{'|'.join(self.cmd_start)}")
        else:
            raise ArgParseError("命令解析器起始符不能和间隔符重合")

    def _purify(self, text: str) -> str:
        """
        处理首尾的空格和行尾序列
        """
        return text.strip(" ").strip("\r\n").strip("\n").strip("\r").strip(" ")

    def _split_string(
        self, string: str, regex: re.Pattern, popFirst: bool = True
    ) -> List[str]:
        """
        按照指定正则 pattern，对 string 进行分割
        """
        # 将复杂的各种分隔符替换为 特殊字符，方便分割
        temp_string = regex.sub("\u0000", string)
        temp_list = re.split("\u0000", temp_string)
        if popFirst:
            temp_list.pop(0)
        return list(filter(lambda x: x != "", temp_list))

    def _parse(self, text: str, textFilter: bool = True) -> Optional[List[List[str]]]:
        pure_string = self._purify(text) if textFilter else text
        cmd_strings = self._split_string(pure_string, self.start_parse_regex)
        cmd_list = [
            self._split_string(s, self.sep_parse_regex, False) for s in cmd_strings
        ]
        cmd_list = list(filter(lambda x: x != [], cmd_list))
        return cmd_list if len(cmd_list) else None

    def parse(self, text: str) -> Optional[Dict[str, ParseArgs]]:
        """
        解析 text
        """
        str_list = self._parse(text)
        if str_list:
            return {
                s[0]: ParseArgs(s[1:]) if len(s) > 1 else ParseArgs(None)
                for s in str_list
            }
        else:
            return None

    def test(
        self, args_group: Dict[str, ParseArgs]
    ) -> Tuple[bool, Optional[str], Optional[ParseArgs]]:
        """
        测试是否匹配。返回三元组：（是否匹配成功，匹配成功的命令名，匹配成功的命令参数）。
        最后两个返回值若不存在，则返回 None
        """
        if args_group is None:
            return (False, None, None)
        if self.target is None:
            return (True, None, None)
        for cmd_name in args_group.keys():
            if cmd_name in self.target:
                return (True, cmd_name, args_group[cmd_name])
        return (False, None, None)

    def format(
        self,
        cutsom_msg_func: Callable[..., Coroutine[Any, Any, "ResponseEvent"]],
        cmd_name: str,
        event: BotEvent,
        args: ParseArgs,
    ) -> bool:
        """
        格式化命令解析参数
        """
        if args.formatted:
            return
        for idx, formatter in enumerate(self.formatters):
            if formatter is None:
                continue
            status = formatter.format(cutsom_msg_func, cmd_name, event, args, idx)
            if not status:
                return False
        args.vals = args.vals[: len(self.formatters)]
        args.formatted = True
        return True


class CmdParserGen:
    """
    命令解析器生成器。预先存储命令起始符和命令间隔符，
    指定匹配 target 后返回一个符合对应匹配条件的命令解析器
    """

    def __init__(self, cmd_start: str | list[str], cmd_sep: str | list[str]) -> None:
        self.cmd_start = cmd_start
        self.cmd_sep = cmd_sep

    def gen(
        self,
        target: str | list[str] = None,
        formatters: List[Optional["ArgFormatter"]] = None,
    ) -> CmdParser:
        """
        生成匹配指定命令的命令解析器
        """
        return CmdParser(self.cmd_start, self.cmd_sep, target, formatters)



import re

from ..types.abc import BotMatcher, LogicMode
from ..types.typing import *


class AlwaysMatch(BotMatcher):
    def __init__(self) -> None:
        super().__init__()

    def match(self, text: str) -> bool:
        return True


class StartMatch(BotMatcher):
    def __init__(self, target: str | list[str], mode: LogicMode = LogicMode.OR) -> None:
        super().__init__()
        self.target = target
        self.mode = mode

    def match(self, text: str) -> bool:
        if isinstance(self.target, str):
            return text[: len(self.target)] == self.target
        else:
            res_seq = [text[: len(s)] == s for s in self.target]
            LogicMode.seq_calc(self.mode, res_seq)


class ContainMatch(BotMatcher):
    def __init__(self, target: str | list[str], mode: LogicMode = LogicMode.OR) -> None:
        super().__init__()
        self.target = target
        self.mode = mode

    def match(self, text: str) -> bool:
        if isinstance(self.target, str):
            return text in self.target
        else:
            res_seq = [text in s for s in self.target]
            LogicMode.seq_calc(self.mode, res_seq)


class EndMatch(BotMatcher):
    def __init__(self, target: str | list[str], mode: LogicMode = LogicMode.OR) -> None:
        super().__init__()
        self.target = target
        self.mode = mode

    def match(self, text: str) -> bool:
        if isinstance(self.target, str):
            return text[-len(self.target) :] == self.target
        else:
            res_seq = [text[-len(s) :] == s for s in self.target]
            LogicMode.seq_calc(self.mode, res_seq)


class FullMatch(BotMatcher):
    def __init__(self, target: str | list[str], mode: LogicMode = LogicMode.OR) -> None:
        super().__init__()
        self.target = target
        self.mode = mode

    def match(self, text: str) -> bool:
        if isinstance(self.target, str):
            return text == self.target
        else:
            res_seq = [text == s for s in self.target]
            LogicMode.seq_calc(self.mode, res_seq)


class RegexMatch(BotMatcher):
    def __init__(self, regex_pattern: str, regex_flags: Any = 0) -> None:
        super().__init__()
        self.pattern = regex_pattern
        self.flag = regex_flags

    def match(self, text: str) -> bool:
        return len(re.findall(self.pattern, text, self.flag)) > 0



import json
from abc import ABC, abstractmethod, abstractproperty
from asyncio import Future
from copy import deepcopy

from .exceptions import *
from .typing import *

if TYPE_CHECKING:
    from ..models.event import ResponseEvent


class AbstractSender(ABC):
    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    async def send(self, action: "BotAction") -> None:
        pass


class AbstractResponder(ABC):
    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    async def dispatch(self, resp: "ResponseEvent") -> None:
        pass

    @abstractmethod
    async def take_action(self, action: "BotAction") -> None:
        pass

    @abstractmethod
    async def take_action_wait(self, action: "BotAction") -> Future["ResponseEvent"]:
        pass


class AbstractDispatcher(ABC):
    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    async def dispatch(self, event: "BotEvent") -> None:
        pass


class Flagable:
    """
    可标记对象
    """

    def __init__(self) -> None:
        self._flags_store: Dict[str, Dict[str, Any]] = None

    def mark(self, namespace: str, flag_name: str, val: Any = None) -> None:
        """
        为对象添加在指定命名空间下，名为 flag_name 的标记。
        此后此对象会一直携带此标记，无法撤销。
        """
        if self._flags_store is None:
            self._flags_store = {}
        if self._flags_store.get(namespace) is None:
            self._flags_store[namespace] = {}
        if flag_name in self._flags_store[namespace].keys():
            raise TryFlagFailed(
                f"对象不可被重复标记。在命名空间 {namespace} 中名为 {flag_name} 的标记已存在"
            )
        self._flags_store[namespace][flag_name] = val

    def flag_check(self, namespace: str, flag_name: str, val: Any = None) -> bool:
        """
        检查此对象是否携带有指定的标记
        """
        self._flags_store = self._flags_store
        if self._flags_store is None:
            return False
        if (flags := self._flags_store.get(namespace)) is None:
            return False
        if (flag := flags.get(flag_name, Void)) is Void:
            return False
        return flag is val if val is None else flag == val


class BotEvent(ABC, Flagable):
    """
    Bot 事件类
    """

    def __init__(self, rawEvent: dict) -> None:
        super().__init__()
        self.raw = rawEvent
        self._args_map: Dict[Any, Dict[str, ParseArgs]] = None

    @abstractproperty
    def time(self) -> int:
        pass

    @abstractproperty
    def type(self) -> str:
        pass

    def is_msg_event(self) -> bool:
        return self.type == "message"

    def is_req_event(self) -> bool:
        return self.type == "request"

    def is_notice_event(self) -> bool:
        return self.type == "notice"

    def is_meta_event(self) -> bool:
        return self.type == "meta"

    def is_resp_event(self) -> bool:
        return self.type == "response"

    def _get_args(self, parser_id: Any) -> dict[str, ParseArgs] | Literal[-1]:
        if self._args_map is None:
            return -1
        return self._args_map.get(parser_id, -1)

    def _store_args(self, parser_id: Any, args_group: dict[str, ParseArgs]) -> None:
        if self._args_map is None:
            self._args_map = {}
        self._args_map[parser_id] = args_group


class ActionArgs(ABC):
    """
    行为信息构造基类
    """

    def __init__(self) -> None:
        super().__init__()
        self.type: str
        self.params: dict


class BotAction(Flagable):
    """
    Bot 行为类
    """

    def __init__(
        self,
        action_args: ActionArgs,
        resp_id: Optional[str] = None,
        triggerEvent: BotEvent = None,
        ready: bool = False,
    ) -> None:
        super().__init__()
        # 只有 action 对应的响应需要被等待单独处理时，才会生成 id
        self.resp_id = resp_id
        self.type = action_args.type
        self.params = action_args.params
        self.trigger = triggerEvent
        self.ready = ready

    def copy(self) -> "BotAction":
        """
        创建当前 action 的深拷贝
        """
        return deepcopy(self)

    def extract(self) -> dict:
        """
        从对象提取标准 cq action dict
        """
        obj = {
            "action": self.type,
            "params": self.params,
        }
        if self.resp_id:
            obj["echo"] = self.resp_id
        return obj

    def flatten(self, indent: int = None) -> str:
        """
        将对象序列化为标准 cq action json 字符串，一般供连接器使用
        """
        return json.dumps(self.extract(), ensure_ascii=False, indent=indent)

    def _fill_trigger(self, event: BotEvent) -> None:
        """
        后期指定触发 event
        """
        if self.trigger is None:
            self.trigger = event
            return
        raise BotActionError("action 已记录触发 event，拒绝再次记录")


class SessionRule(ABC):
    """
    用作 sesion 的区分依据
    """

    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    def compare(self, e1: BotEvent, e2: BotEvent) -> bool:
        pass


class ShareObjArgs:
    """
    插件共享对象构造参数
    """

    def __init__(self, property: str, namespace: str, id: str) -> None:
        self.property = property
        self.namespace = namespace
        self.id = id


class ShareObjCbArgs:
    """
    插件共享对象回调的构造参数
    """

    def __init__(
        self, namespace: str, id: str, cb: Callable[..., Coroutine[Any, Any, Any]]
    ) -> None:
        self.namespace = namespace
        self.id = id
        self.cb = cb


class PluginSignalHandlerArgs:
    """
    插件信号方法构造参数
    """

    def __init__(
        self, func: Callable[..., Coroutine[Any, Any, Any]], namespace: str, signal: str
    ) -> None:
        self.func = func
        self.namespace = namespace
        self.signal = signal


class BotHookRunnerArgs:
    """
    钩子方法（生命周期回调）构造参数
    """

    def __init__(
        self, func: Callable[..., Coroutine[Any, Any, None]], type: BotLife
    ) -> None:
        self.func = func
        self.type = type


class BotChecker(ABC):
    def __init__(self) -> None:
        super().__init__()

    def __and__(self, other: "BotChecker") -> "WrappedChecker":
        if not isinstance(other, BotChecker):
            raise BotCheckerError(
                f"联合检查器定义时出现了非检查器对象，其值为：{other}"
            )
        return WrappedChecker(LogicMode.AND, self, other)

    def __or__(self, other: "BotChecker") -> "WrappedChecker":
        if not isinstance(other, BotChecker):
            raise BotCheckerError(
                f"联合检查器定义时出现了非检查器对象，其值为：{other}"
            )
        return WrappedChecker(LogicMode.OR, self, other)

    def __invert__(self) -> "WrappedMatcher":
        return WrappedChecker(LogicMode.NOT, self)

    def __xor__(self, other: "BotChecker") -> "WrappedChecker":
        if not isinstance(other, BotChecker):
            raise BotCheckerError(
                f"联合检查器定义时出现了非检查器对象，其值为：{other}"
            )
        return WrappedChecker(LogicMode.XOR, self, other)

    @abstractmethod
    def check(self, event: BotEvent) -> bool:
        pass


class WrappedChecker(BotChecker):
    """
    按逻辑关系工作的的合并检查器，使用 AND, OR, XOR 模式时，
    需要传递两个 checker。使用 NOT 时只需要传递第一个 checker
    """

    def __init__(
        self, mode: LogicMode, checker1: BotChecker, checker2: BotChecker = None
    ) -> None:
        super().__init__()
        self.mode = mode
        self.c1, self.c2 = checker1, checker2

    def check(self, event: BotEvent) -> bool:
        return LogicMode.calc(
            self.mode,
            self.c1.check(event),
            self.c2.check(event) if self.c2 is not None else None,
        )


class BotMatcher(ABC):
    def __init__(self) -> None:
        super().__init__()

    def __and__(self, other: "BotMatcher") -> "WrappedMatcher":
        if not isinstance(other, BotMatcher):
            raise BotMatcherError(
                f"联合匹配器定义时出现了非匹配器对象，其值为：{other}"
            )
        return WrappedMatcher(LogicMode.AND, self, other)

    def __or__(self, other: "BotMatcher") -> "WrappedMatcher":
        if not isinstance(other, BotMatcher):
            raise BotMatcherError(
                f"联合匹配器定义时出现了非匹配器对象，其值为：{other}"
            )
        return WrappedMatcher(LogicMode.OR, self, other)

    def __invert__(self) -> "WrappedMatcher":
        return WrappedMatcher(LogicMode.NOT, self)

    def __xor__(self, other: "BotMatcher") -> "WrappedMatcher":
        if not isinstance(other, BotMatcher):
            raise BotMatcherError(
                f"联合匹配器定义时出现了非匹配器对象，其值为：{other}"
            )
        return WrappedMatcher(LogicMode.XOR, self, other)

    @abstractmethod
    def match(self, text: str) -> bool:
        pass


class WrappedMatcher(BotMatcher):
    """
    按逻辑关系工作的的合并匹配器，使用 AND, OR, XOR 模式时，
    需要传递两个 matcher。使用 NOT 时只需要传递第一个 matcher
    """

    def __init__(
        self, mode: LogicMode, matcher1: BotMatcher, matcher2: BotMatcher = None
    ) -> None:
        super().__init__()
        self.mode = mode
        self.m1, self.m2 = matcher1, matcher2

    def match(self, text: str) -> bool:
        return LogicMode.calc(
            self.mode,
            self.m1.match(text),
            self.m2.match(text) if self.m2 is not None else None,
        )


class BotParser(ABC):
    """
    解析器基类。解析器一般用作从消息文本中按规则提取指定字符串或字符串组合
    """

    def __init__(self, id: Any) -> None:
        super().__init__()
        self.id = id
        self.need_format: bool = False

    @abstractmethod
    def parse(self, text: str) -> Optional[Dict[str, ParseArgs]]:
        pass

    @abstractmethod
    def test(
        self, args_group: Dict[str, ParseArgs]
    ) -> Tuple[bool, Optional[str], Optional[ParseArgs]]:
        pass

    @abstractmethod
    def format(
        self,
        custom_msg_func: Callable[..., Coroutine[Any, Any, "ResponseEvent"]],
        cmd_name: str,
        event: BotEvent,
        args: ParseArgs,
    ) -> bool:
        pass
